import ActiveDirectory from 'activedirectory'
import AES from '../../../lib/aes'
import { Base64 as DefaultPath} from 'js-base64';

export default async function userHandler(req, res) {

    const {
        query: params,
        method,
    } = req

    let postData = req.body;
    console.log(postData)

    let config
    if (postData.url) {
        config = { 
            url: postData.url + ':' + postData.port,
            baseDN: postData.baseDN,
            username: postData.user + '@' + postData.url.substring(7),
            password: postData.pass
        }
    } else {
        const serviceURL = process.env.SERVICE_URL;
        const result = await fetch(serviceURL + '/ldapsettings')
        const encodedResult = await result.text()  
        const ldapSettings = JSON.parse(AES.Decrypt(encodedResult, DefaultPath.decode(process.env.JAKO), DefaultPath.decode(process.env.OJAK)))    
        console.log(ldapSettings)
        config = { 
            url: ldapSettings.url + ':' + ldapSettings.port,
            baseDN: ldapSettings.baseDN,
            username: ldapSettings.user,
            password: ldapSettings.pass
        }
    }

    console.log(config)
    var ad = new ActiveDirectory(config);

    switch (method) {
      case 'POST':    
        // Authenticate user          
        ad.authenticate(postData.user, postData.pass, function(err, auth) {
            if (err) {
                console.log('ERROR: '+JSON.stringify(err));
                res.status(500).end();
                return;
            }
            
            if (auth) {
                console.log('Authenticated!');
                res.status(200).end();              
            }
            else {
                console.log('Authentication failed!');
                res.status(404).end();              
            }
        });

        break
      default:
        res.setHeader('Allow', ['POST'])
        res.status(405).end(`Method ${method} Not Allowed`)        
    }
}