

import { Alert, Container,Row, Col, Badge} from 'react-bootstrap'
import dynamic from 'next/dynamic'
import { withIronSession } from "next-iron-session";
import AES from '../lib/aes'
import React from 'react';
import SuspendRestrictions from './layouts/SuspendRestrictions'
import { Base64 as DefaultPath} from 'js-base64';

const Header = dynamic(() => import('./layouts/Header'))
const RestrictionsSummary = dynamic(() => import('./layouts/RestrictionsSummary'))
const SystemKeysSummary = dynamic(() => import('./layouts/SystemKeysSummary'))
const UserDetails = dynamic(() => import('./layouts/UserDetails'))
const ComputerDetails = dynamic(() => import('./layouts/ComputerDetails'))

export const getServerSideProps = withIronSession(
	async ({ req, res }) => {
		// log module
		const logModule = 'dashboard';
		const user = req.session.get("user");
		const permissions = req.session.get("permissions");		
		console.log(req.session.get())
		if (!user) { 
			return { 
				redirect: {
				  destination: '/',
				  permanent: false,
				}
			  };
		} else if (permissions.permissions.indexOf(logModule) < 0 ) {
			res.statusCode = 403;
			res.end();
			return { props: {} };
		} else {			
			// fetch data from config file and create state for all the controllers
			require('dotenv').config()
			const serviceURL = process.env.SERVICE_URL;
			const JAKO = DefaultPath.decode(process.env.JAKO);
			const OJAK = DefaultPath.decode(process.env.OJAK);
			const locale = process.env.locale

			let regedit = require('regedit')
			await regedit.arch.list64('HKLM\\SOFTWARE\\LabLock\\KAgentService')
			.on('data', function(entry) {
			  process.env.PORT = entry.data['values']['Port']['value']
			  var os = require("os");
			  var hostname = os.hostname();
			  process.env.SERVICE_URL = 'http://' + hostname + ':' + entry.data['values']['Port']['value']
			  console.log(process.env.PORT)
			})
			.on('finish', function () {
			  console.log('list operation finished')
			})
			.on('error', function () {
			  console.log('Not a Windows OS')
			})    
			console.log(process.env.SERVICE_URL)
			// retrieve summary details
			const res0 = await fetch(serviceURL + '/summary')
			var resEncoded = await res0.text()
			const summaryDetails = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
			console.log(summaryDetails)			
			// Retrieve computer details
			const res1 = await fetch(serviceURL + '/computerdetails')
			var resEncoded = await res1.text()
			const computerDetails = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
			console.log(computerDetails)
			// Retrieve users logged in
			const res2 = await fetch(serviceURL + '/logonuserdetails')
			resEncoded = await res2.text()
			const logonUserDetailsList = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
			// Retrieve restrictions
			const res4 = await fetch(serviceURL + '/settings')
			resEncoded = await res4.text()
			const restrictions = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
			let restrictionsCount = restrictions.settings.length

			const res6 = await fetch(serviceURL + '/syskeys')
			resEncoded = await res6.text()
			const systemKeys = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
			restrictionsCount += systemKeys.SystemKeys.length

			const res7 = await fetch(serviceURL + '/appblocker')
			resEncoded = await res7.text()
			const apps = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))		
			restrictionsCount += apps.apps.length
			// Retrieve if there are any restrictions or restrictions supension time
			const res3 = await fetch(serviceURL + '/suspend')
			resEncoded = await res3.text()
			const suspendProperties = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
			console.log(suspendProperties)
			// Retrieve suspension time
			const res5 = await fetch(serviceURL + '/lablocksettings')
			resEncoded = await res5.text()
			const generalSettings = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
			suspendProperties['duration'] = generalSettings['suspendRestrictionsTime']
			process.env.SESSION_TIMEOUT = generalSettings['sessionExpiration']

			// console.log("u:" + user.user.username)
			const userObj = user.user

			return {
				props: {
					summaryDetails,
					logonUserDetailsList,
					computerDetails, 
					serviceURL,
					logModule,
					suspendProperties,
					restrictionsCount,
					userObj,
					permissions,
					JAKO,
					OJAK,
					locale
				}
			}
		}
	},
    {
		cookieName: process.env.SESSION_COOKIE,
		cookieOptions: {
			secure: process.env.NODE_ENV === "production" ? true : false,
			maxAge: process.env.SESSION_TIMEOUT * 60
		},
		password: process.env.APPLICATION_SECRET
	}
);


const Index = ( props ) => (
  <>
    
	
	<Header user={props.userObj} permissions={props.permissions} serviceURL={props.serviceURL} JAKO={props.JAKO} OJAK={props.OJAK}  />

    <Container >
		
		<table width="100%">
			<tbody>
			<tr>
				<td colSpan="3">
				<Alert variant="info">				
					<h1>Dashboard</h1>
					<p>
						<Badge  variant="dark">
							<em>LabLock - Version: {props.computerDetails.LabLockVersion}</em>
						</Badge>
					</p>
				</Alert>				
				</td>
			</tr>
			<tr>
				<td colSpan="3">
				<table width="100%">
					<tbody>
					<tr>
						<td>				
						<ComputerDetails {...props}/>  				
						</td>
						<td valign="top" >
						<SuspendRestrictions {...props}/>  
						</td>
					</tr>
					</tbody>
				</table>
				</td>
			</tr>
			<tr><td colSpan="3">&nbsp;</td></tr>
			<tr>
				<td colSpan="3">
					<Alert variant="secondary">	
						<table>
							<tbody>
								<tr>
									<td >
										<svg width="3em" height="3em" viewBox="0 0 16 16" className="bi bi-people" fill="currentColor" xmlns="http://www.w3.org/2000/svg">
											<path fillRule="evenodd" d="M15 14s1 0 1-1-1-4-5-4-5 3-5 4 1 1 1 1h8zm-7.995-.944v-.002.002zM7.022 13h7.956a.274.274 0 0 0 .014-.002l.008-.002c-.002-.264-.167-1.03-.76-1.72C13.688 10.629 12.718 10 11 10c-1.717 0-2.687.63-3.24 1.276-.593.69-.759 1.457-.76 1.72a1.05 1.05 0 0 0 .022.004zm7.973.056v-.002.002zM11 7a2 2 0 1 0 0-4 2 2 0 0 0 0 4zm3-2a3 3 0 1 1-6 0 3 3 0 0 1 6 0zM6.936 9.28a5.88 5.88 0 0 0-1.23-.247A7.35 7.35 0 0 0 5 9c-4 0-5 3-5 4 0 .667.333 1 1 1h4.216A2.238 2.238 0 0 1 5 13c0-1.01.377-2.042 1.09-2.904.243-.294.526-.569.846-.816zM4.92 10c-1.668.02-2.615.64-3.16 1.276C1.163 11.97 1 12.739 1 13h3c0-1.045.323-2.086.92-3zM1.5 5.5a3 3 0 1 1 6 0 3 3 0 0 1-6 0zm3-2a2 2 0 1 0 0 4 2 2 0 0 0 0-4z"/>
										</svg>
									</td>
									<td>
										<h2>User details</h2>
									</td>
								</tr>
							</tbody>
						</table>							 					
					</Alert>
				</td>
			</tr>						
			<tr>
				<td colSpan="3">
					<UserDetails sessionList={props.logonUserDetailsList.sessions} serviceURL={props.serviceURL} user={props.userObj} JAKO={props.JAKO} OJAK={props.OJAK} computerDetails={props.computerDetails} locale={props.locale} />
				</td>
			</tr>	
			<tr><td colSpan="3">&nbsp;</td></tr>
			<tr><td colSpan="3">&nbsp;</td></tr>				
			<tr>
				<td colSpan="3">
				{(props.summaryDetails.Summary.Restrictions.Totals[0].value > 0) ? <RestrictionsSummary {...props}/> : ''}
				</td>
			</tr>
			<tr>
				<td colSpan="3">
				{(props.summaryDetails.Summary.SystemKeys.Totals[1].value > 0) ? <SystemKeysSummary {...props}/> : ''}
				</td>
			</tr>
			</tbody>
		</table>						
		
    </Container>

  </>  
)

export default Index