import Header from './layouts/Header';
import ImportExportContainer from './layouts/ImportExportContainer';
import { withIronSession } from "next-iron-session";
import { Alert, Container, Badge} from 'react-bootstrap'
import AES from '../lib/aes'
import { Base64 as DefaultPath} from 'js-base64';

export const getServerSideProps = withIronSession(
	async ({ req, res }) => {
		// log module
		const logModule = 'import_export';
		var user = req.session.get("user");
		var permissions = req.session.get("permissions");

		if (!user) { 
			return { 
        redirect: {
          destination: '/',
          permanent: false,
        }
      };
    } else if (permissions.permissions.indexOf(logModule) < 0 ) {
			res.statusCode = 403;
			res.end();
			return { props: {} };
    } else {
      // fetch data from config file and create state for all the controllers
			require('dotenv').config()
			const serviceURL = process.env.SERVICE_URL;
			const JAKO = DefaultPath.decode(process.env.JAKO);
      const OJAK = DefaultPath.decode(process.env.OJAK);
      
      let regedit = require('regedit')
      await regedit.arch.list64('HKLM\\SOFTWARE\\LabLock\\KAgentService')
      .on('data', function(entry) {
        process.env.PORT = entry.data['values']['Port']['value']
        var os = require("os");
        var hostname = os.hostname();
        process.env.SERVICE_URL = 'http://' + hostname + ':' + entry.data['values']['Port']['value']
        console.log(process.env.PORT)
      })
      .on('finish', function () {
        console.log('list operation finished')
      })
      .on('error', function () {
        console.log('Not a Windows OS')
      })    
      console.log(process.env.SERVICE_URL)
            
			const userObj = user.user

      // Retrieve computer details
			let res1 = await fetch(serviceURL + '/computerdetails')
			let resEncoded = await res1.text()
			let computerDetails = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))

      return {
        props: { 
          serviceURL,
          userObj,
          logModule,
          computerDetails,
          permissions,
          JAKO,
          OJAK
        }
      }
    }
  },
    {
      cookieName: process.env.SESSION_COOKIE,
      cookieOptions: {
        secure: process.env.NODE_ENV === "production" ? true : false,
        maxAge: process.env.SESSION_TIMEOUT * 60
      },
      password: process.env.APPLICATION_SECRET
    }
  );

const Index = ( props ) => (
  <>
    <Header user={props.userObj} permissions={props.permissions} serviceURL={props.serviceURL} JAKO={props.JAKO} OJAK={props.OJAK} />

    <Container>
      <Alert variant="info">				
        <h1>Import / Export</h1>
        <p>
					<Badge  variant="dark">
  					<em>LabLock - Version: {props.computerDetails.LabLockVersion}</em>
					</Badge>
        </p>
     </Alert>
    
      <ImportExportContainer {...props}/>
    </Container>
  </>  
)

export default Index