import Button from 'react-bootstrap/Button'
import Container from 'react-bootstrap/Container'
import Jumbotron from 'react-bootstrap/Jumbotron'
import Form from 'react-bootstrap/Form'
import Row from 'react-bootstrap/Row'
import Col from 'react-bootstrap/Col'
import ListGroup from 'react-bootstrap/ListGroup'
import Modal from 'react-bootstrap/Modal'
import Datetime from 'react-datetime';
import { Formik } from 'formik';
import * as yup from 'yup';
// import { CSVLink } from 'react-csv';
import { jsPDF } from 'jspdf';
import { ExportToCsv } from 'export-to-csv';
import { toTimestamp } from '../../lib/date'
import log from '../../lib/log'
// import 'moment/locale/en-ie';
import dynamic from 'next/dynamic'
// import ActivityDetailsModal from './ActivityDetailsModal'
const ActivityDetailsModal = dynamic(() => import('./ActivityDetailsModal'))


class ActivityContainer extends React.Component {
	constructor(props) {
		super(props);

        let toDate = new Date().toLocaleDateString(props.locale) + " " + new Date().toLocaleTimeString(props.locale);
        let fromDate = new Date();
        fromDate.setHours(fromDate.getHours()-24)
        fromDate = fromDate.toLocaleDateString(props.locale) + " " + fromDate.toLocaleTimeString(props.locale) ;

        let schema = yup.object({
            // dateFrom: yup.date().required(),
            // dateTo: yup.date().required(),
            username: yup.string(),
            computer: yup.string()
        })

        this.state = {
            showErrorModal: false,
            loglines: [],
            loglinesDisplayed: [],
            inputPropsDateTo: {placeholder: "Date To"},
            inputPropsDateFrom: {placeholder: "Date From"},
            initialFromDate: fromDate,
            initialToDate: toDate,
            showResults: 50,
            format: 'csv',
            size: 'full',
            schema: schema,
            searchForm: {dateTo: toDate, dateFrom: fromDate, username: '', computer: ''}
        };

        // bind the action handlers
        this.handleSearch = this.handleSearch.bind(this);
        this.changeShowResults = this.changeShowResults.bind(this);
        this.generateReport = this.generateReport.bind(this);
        this.handleExportShow = this.handleExportShow.bind(this);
        this.handleExportClose = this.handleExportClose.bind(this);
        this.handleExportSize = this.handleExportSize.bind(this)
        this.handleExportFormat = this.handleExportFormat.bind(this)
        this.handleFromDateChange = this.handleFromDateChange.bind(this)
        this.handleToDateChange = this.handleToDateChange.bind(this)

        this.loadLog()
    }

	componentDidMount(){
		var getUrl = window.location;		
		console.log(getUrl.host)
		var hostIP = getUrl.host
		this.setState(state => ({host: hostIP}))
    }
        
    loadLog(){
        fetch('/api/activity')
        .then(res => res.json())
        .then(data => {            
            this.setState(state => ({loglines: data.activity}))
console.log(this.props.locale)
            let counter = 1
            let loglinesDisplayed = []
            data.activity.map((line) => {
                line['date'] = new Date(line['date']).toLocaleDateString(this.props.locale) + " " + new Date(line['date']).toLocaleTimeString(this.props.locale) 
                loglinesDisplayed.push(line)                
                if (counter == this.state.showResults)
                    return
                counter++;
            })

            this.setState(state => ({loglinesDisplayed: loglinesDisplayed}))
            
            // console.log(data.activity)
        }) 
        .catch(err => console.log(err));
    }

    handleFromDateChange(e) {
        console.log(e)       
        if (typeof e == "object") {
            this.setState(state => ({fromDate: new Date(e.unix()*1000).toLocaleDateString(this.props.locale) + " " + new Date(e.unix()*1000).toLocaleTimeString(this.props.locale) }))
        }
    }

    handleToDateChange(e) {
        console.log(e)
        if (typeof e == "object") {
            this.setState(state => ({toDate: new Date(e.unix()*1000).toLocaleDateString(this.props.locale) + " " + new Date(e.unix()*1000).toLocaleTimeString(this.props.locale) }))
        }        
    }

    handleSearch(data) {
        
        // transform checkboxes value to integer
        let modules = []
        data.login ? modules.push("login") : ''
        data.dashboard ? modules.push("dashboard") : ''
        data.restrictions ? modules.push("restrictions") : ''
        data.settings ? modules.push("settings") : ''
        data.import_export ? modules.push("import_export") : ''
        // console.log(modules)

        // transform dates depending if they are  the initial ones or selected ones
        let dateFrom
        if (!this.state.fromDate) {
            // console.log(this.state.initialFromDate)
            // let datetimeParts = this.state.initialFromDate.split(' ')
            // let dateParts = datetimeParts[0].split('/')
            // let formattedFromDate = ('0'+dateParts[0]).slice(-2)+'/'+('0'+dateParts[1]).slice(-2)+'/'+dateParts[2]+' '+datetimeParts[1]
            // console.log(formattedFromDate)
            let formattedFromDate = this.state.initialFromDate
            dateFrom = toTimestamp(formattedFromDate, this.props.locale)
        } else {
            console.log(this.state.fromDate)
            dateFrom = toTimestamp(this.state.fromDate, this.props.locale)
        }
        
        let dateTo
        if (!this.state.toDate) {
            // console.log(this.state.initialToDate)
            // let datetimeParts = this.state.initialToDate.split(' ')
            // let dateParts = datetimeParts[0].split('/')
            // let formattedToDate = ('0'+dateParts[0]).slice(-2)+'/'+('0'+dateParts[1]).slice(-2)+'/'+dateParts[2]+' '+datetimeParts[1]
            let formattedToDate = this.state.initialToDate
            dateTo = toTimestamp(formattedToDate, this.props.locale)
        } else {
            dateTo = toTimestamp(this.state.toDate, this.props.locale)
        }
        
        console.log(dateFrom)
        console.log(dateTo)

        let counter = 0
        let loglinesDisplayed = []
        this.state.loglines.map((line) => {
            // console.log(line.date)
            let dateLine = toTimestamp(line.date, this.props.locale) 
            // console.log(dateLine)
            if (dateLine >= dateFrom  && dateLine <= dateTo) {
                if (data.computer != '' && data.username != '') {
                    // console.log("user.computer")
                    if (data.computer == line.computer && data.username == line.user) {
                        if (modules.length == 0 || modules.includes(line.module)) {
                            loglinesDisplayed.push(line);
                            counter++;
                        }
                    }
                } else if (data.computer != '') {
                    // console.log("data.computer")
                    if (data.computer == line.computer) {
                        if (modules.length == 0 || modules.includes(line.module)) {
                            loglinesDisplayed.push(line);
                            counter++;
                        }
                    }
                } else if (data.username != '') {
                    // console.log(line.user)
                    if (data.username == line.user) {
                        if (modules.length == 0 || modules.includes(line.module)) {
                            loglinesDisplayed.push(line);
                            counter++;       
                        }
                    }
                } else {
                    // console.log(modules.length)
                    if (modules.length == 0 || modules.includes(line.module)) {
                        loglinesDisplayed.push(line);
                        counter++;
                    }
                }
            }

            if (counter == this.state.showResults)
                return
        })
        console.log(loglinesDisplayed)

        this.setState(state => ({loglinesDisplayed: loglinesDisplayed}))
    }

    handleExportClose() {
        this.setState({showExportModal: false})
    }

    handleExportShow() {
        this.setState({showExportModal: true})  
    }   

    handleErrorClose() {
        this.setState({showErrorModal: false})
    }

    handleErrorShow() {
        this.setState({showErrorModal: true})  
    }     

    changeShowResults(e) {
        let value = e.target.value
        this.setState(state => ({showResults: value}))
    }

    handleExportFormat(e){
        let value = e.target.value
        this.setState(state => ({ format: value}))
    }

    handleExportSize(e){
        let value = e.target.value
        this.setState(state => ({ size: value}))
    }

    generateReport() {

        let exportLines = []
        switch (this.state.format) {
            case 'csv':
                const options = { 
                    fieldSeparator: ',',
                    filename: 'lablockExport',
                    quoteStrings: '"',
                    decimalSeparator: '.',
                    showLabels: true, 
                    showTitle: true,
                    title: 'LablockExport',
                    useTextFile: false,
                    useBom: true,
                    // useKeysAsHeaders: true,
                  };

                
                switch (this.state.size) {
                    case 'full':
                        exportLines = this.state.loglines;
                        break;
                    case 'partial':
                        exportLines = this.state.loglinesDisplayed;
                        break;                     
                }                  
                let csvExporter = new ExportToCsv(options);                 
                csvExporter.generateCsv(exportLines);

                break;

            case 'pdf':
                console.log(this.state.format)
                let doc = new jsPDF('p', 'pt');
                doc.text("Lablock Export", 20, 20);      
                switch (this.state.size) {
                    case 'full':
                        this.state.loglines.map((line) => {
                            exportLines.push(line.date +' '+ line.user +' '+ line.description +' '+ line.computer);
                        })
                        break;
                    case 'partial':
                        this.state.loglinesDisplayed.map((line) => {
                            exportLines.push(line.date +' '+ line.user +' '+ line.description +' '+ line.computer);
                        })   
                        break;                     
                }
                doc.setFontSize(10)
                doc.text(exportLines, 15, 40)
                doc.save("lablockExport.pdf");                
                break;
            default:
                break;
        }

        log(this.props.user.username, this.props.logModule, "export activity log file in "+ this.state.format +" format from ", this.state.host)
        this.handleExportClose()
    }

    render(){
        return (
            <>                 

                <Jumbotron>
                    <Container>
                        <Formik
                                validationSchema={this.state.schema}
                                onSubmit={(values) => {
                                    console.log(values)
                                    this.handleSearch(values);
                                }}
                                initialValues={this.state.searchForm}
                            >
                            {({
                            handleSubmit,
                            handleChange,
                            handleBlur,
                            values,
                            touched,
                            isValid,
                            errors,
                            }) => (
                            <Form onSubmit={handleSubmit}>                         
                                <Form.Row>
                                    <Col sm={3}>
                                        <Form.Control id="username" name="username" value={values.username} onChange={handleChange} placeholder="username"/>
                                    </Col>   
                                    <Col sm={3}>
                                        <Form.Control id="computer" name="computer" value={values.computer} onChange={handleChange} placeholder="Computer name / IP" />
                                    </Col>                                                                     
                                    <Col sm={3}>
                                        {/* <Form.Control placeholder="Date from" id="dateFrom"/> */}
                                        <Datetime id="dateFrom" name="dateFrom" onChange={this.handleFromDateChange} inputProps={this.state.inputPropsDateFrom} 
                                         initialValue={this.state.initialFromDate} closeOnSelect="true" locale={this.props.locale}/>
                                    </Col>
                                    <Col sm={3}>
                                        {/* <Form.Control placeholder="Date to" id="dateTo"/> */}
                                        <Datetime id="dateTo" name="dateTo" onChange={this.handleToDateChange} inputProps={this.state.inputPropsDateTo} 
                                         initialValue={this.state.initialToDate} closeOnSelect="true" locale={this.props.locale}/>
                                    </Col>                                    
                                </Form.Row>         
                                {/*
                                <br/>            

                                 <Form.Row>
                                    <Col sm={8}>
                                        <Form.Control id="description" placeholder="Description" />
                                    </Col>
                                    <Col sm={4}>
                                        <Form.Control id="computer" placeholder="Computer name / IP" />
                                    </Col>
                                </Form.Row>   */}

                                <br />  

                                <Form.Row>
                                    <Form.Check inline id="login" value={values.login} onChange={handleChange} type="checkbox" label="Login   " />
                                    <Form.Check inline id="dashboard" value={values.dashboard} onChange={handleChange} type="checkbox" label="Dashboard   " />
                                    <Form.Check inline id="restrictions" value={values.restrictions} onChange={handleChange} type="checkbox" label="Restrictions   " />
                                    <Form.Check inline id="settings" value={values.settings} onChange={handleChange} type="checkbox" label="Settings" />
                                    <Form.Check inline id="import_export" value={values.import_export} onChange={handleChange} type="checkbox" label="Import / Export" />
                                    <Col md={{ span: 1, offset: 6 }}>
                                        <Button id="btn-search" type="submit" variant="dark" >
                                            Search
                                        </Button>	
                                    </Col>	                                    
                                </Form.Row>  

                            </Form>
                        )}
                        </Formik>

                        <br />

                        <ListGroup>
                        <ListGroup.Item variant="info">
                            <Form.Row>
                                <Col md={{ span: 2, offset: 0 }}>
                                    Recent Activity
                                </Col>
                                <Col md={{ span: 1, offset: 9 }}>
                                    <Button id="btn-generate-export" variant="link" onClick={this.handleExportShow}>Export</Button>    
                                </Col>                                                        
                            </Form.Row>
                        </ListGroup.Item>
                        {this.state.loglinesDisplayed.map((line, index) => 
                            index < this.state.showResults && line.details ?
                            <ListGroup.Item key={index}>{line.date} {line.user} {line.description} {line.computer} {" "}
                                <ActivityDetailsModal details={line.details}/>
                            </ListGroup.Item>                   
                        :
                            <ListGroup.Item key={index}>{line.date} {line.user} {line.description} {line.computer}</ListGroup.Item>
                        )}                        
                        </ListGroup>
                        <Form.Row>
                            <Button id="btn-show50" value="50" variant="link" size="sm" onClick={this.changeShowResults}>50</Button>|
                            <Button id="btn-show200" value="200" variant="link" size="sm" onClick={this.changeShowResults}>200</Button>|
                            <Button id="btn-show1000" value="1000" variant="link" size="sm" onClick={this.changeShowResults}>1000</Button>
                            {/* <Col md={{ span: 0, offset: 3 }}>
                                <CSVLink data={this.state.loglinesDisplayed}>Download selected</CSVLink> |
                                <CSVLink data={this.state.loglines}>Download all</CSVLink>                                
                            </Col>      
                            <Col md={{ span: 2, offset: 2 }}>*/}
                            <Col md={{ span: 0, offset: 9 }}>
                                <Form.Label>Total records: {this.state.loglinesDisplayed.length}</Form.Label>
                            </Col>                                                              
                        </Form.Row> 

                        {/* <Form.Row>
                            <Col md={{ span: 0, offset: 11 }}>
                                
                            </Col>
                        </Form.Row> */}

                        <Modal show={this.state.showExportModal} onHide={this.handleExportClose} aria-labelledby="contained-modal-title-vcenter" centered>
                            <Modal.Header closeButton>
                                <Modal.Title>Export generation</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>                            
                                <Form.Group as={Row} controlId="format">
                                    <Form.Label column sm="3">Format</Form.Label>
                                    <Col sm="6">
                                        <Form.Control as="select" defaultValue={this.state.format} onChange={this.handleExportFormat} custom >
                                            <option value="csv">CSV</option>
                                            {/* <option value="pdf">PDF</option> */}
                                        </Form.Control>
                                    </Col>
                                </Form.Group>

                                <Form.Group as={Row} controlId="size">
                                    <Form.Label column sm="3">Export size</Form.Label>
                                    <Col sm="6">
                                        <Form.Control as="select" defaultValue={this.state.size} onChange={this.handleExportSize} custom >
                                            <option value="full">Full log</option>
                                            <option value="partial">Filtered log</option>
                                        </Form.Control>
                                    </Col>
                                </Form.Group>

                                <Col md={{ span: 0, offset: 4 }}>
                                    <Button variant="success" onClick={this.generateReport}>Generate</Button>{' '}
                                    <Button variant="secondary" onClick={this.handleExportClose}>Close</Button>
                                </Col>                          
                            </Modal.Body>  

                            {/* <Modal.Footer>
                                <Button variant="success" onClick={this.generateReport}>Generate</Button>
                                <Button variant="secondary" onClick={this.handleExportClose}>Close</Button>
                            </Modal.Footer>                         */}
                        </Modal>


                        <Modal show={this.state.showErrorModal} onHide={this.handleErrorClose} aria-labelledby="contained-modal-title-vcenter" centered>
                            <Modal.Header>
                                <Modal.Title>Ops, something didn't work as expected</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>There has been an error when processing your request, please try again or contact support</Modal.Body>
                            <Modal.Footer>
                                <Button variant="secondary" onClick={this.handleErrorClose}>
                                Close
                                </Button>
                            </Modal.Footer>
                        </Modal>                         
                    </Container>
                </Jumbotron>
            </>
        );
  } 
}

export default ActivityContainer;