import Form from 'react-bootstrap/Form'
import Container from 'react-bootstrap/Container'
import Jumbotron from 'react-bootstrap/Jumbotron'
import OverlayTrigger from 'react-bootstrap/OverlayTrigger'
import Tooltip from 'react-bootstrap/Tooltip'
import InputGroup from 'react-bootstrap/InputGroup'
import Card from 'react-bootstrap/Card'
import Accordion from 'react-bootstrap/Accordion'
import Button from 'react-bootstrap/Button'
import dynamic from 'next/dynamic'
import LockPcRestriction from './LockPcRestriction'
const DriveSelection = dynamic(() => import('./DriveSelectionRestriction'))
const FileExplorerPathRestriction = dynamic(() => import('./FileExplorerPathRestriction'))

class Desktop extends React.Component {
	constructor(props) {
		super(props);

		// initialize special fields
		let filePath = ''
		let lockPC = ''
		if (this.props.restrictions != null) {
			if (this.props.restrictions.settings){
				this.props.restrictions.settings.map((setting) => {
					console.log(setting.tag)
					console.log(setting.value)
					if (setting.tag == "SYS_AddFileExplorerPath") {
						filePath = setting.value
					}			
					if (setting.tag  == "SYS_LockPC") {
						lockPC = setting.value/60
					}
				});	
			}
		} else {
			this.props.components = []
		}
		let sections = {sections: []}
		this.state = {
			componentsUI: sections,
			filePath: filePath,
			lockPC: lockPC
		}
		this.handleChange = this.handleChange.bind(this);
		console.log(this.state)
	}

	componentDidMount(){
		let componentsUI = {}
		componentsUI.sections = []
		let section
		// section.items = []
		let restrictionArea = ''
		let items = []
		this.props.components.configs.map((component) => {
			if (restrictionArea == '')			
				restrictionArea = component.restrictionArea
			
			if (restrictionArea != component.restrictionArea ) {				
				// console.log(items)
				section = {label: restrictionArea, "items": items}
				componentsUI.sections.push(section)
				// console.log(componentsUI)
				// reinitialize for next section
				restrictionArea = component.restrictionArea
				items = []				
			}
			items.push(component)

		})
		// add last group
		section = {label: restrictionArea, "items": items}
		componentsUI.sections.push(section)
		console.log(componentsUI)

		this.setState(state => ({componentsUI: componentsUI}))
		// console.log(componentsUI)
    }	

	checkStatus(searchVal) {

		let checked = false
		this.props.restrictions.settings.map((restriction) => {
			if (restriction.tag == searchVal)
				checked = true;
		});
		return checked;
	}

	addLabel(searchVal) {

		let label
		this.props.components.configs.map((component) => {
			if (component.restrictionTag == searchVal)
				label = component.restrictionLabel
			return label;
		}
		);
		return label;
	}

	addTooltip(searchVal) {

		let description
		this.props.components.configs.map((component) => {
			//console.log(component);
			if (component.restrictionTag == searchVal)
				description = component.restrictionDesc
			return description;
		}
		);
		//console.log(checked);
		return description;
	}

	handleChange(e) {
		this.props.handleRestrictionsChange(e)
		if (e.target.id == "SYS_AddFileExplorerPathCheck" && e.target.checked == true)  {
			this.setState(state => ({showPathTextbox: true}))
		} else if (e.target.id == "SYS_AddFileExplorerPathCheck" && e.target.checked == false)  {
			this.setState(state => ({showPathTextbox: false}))
		}		
	}

	checkStatusToogleGroup(searchVal) {
		let defaultValues = []
		this.props.restrictions.settings.map((restriction) => {
			defaultValues.push(restriction.tag);
		});
		//console.log(defaultValues);
		return defaultValues;
	}

	render() {

		return (
			<>
				<Jumbotron>
					<Container>
						<Accordion defaultActiveKey="Desktop">
						{ this.state.componentsUI.sections.map((section, index) => 							
							<Card>
								<Card.Header>
									<Accordion.Toggle as={Button} variant="link" eventKey={section.label}>
										{section.label}
									</Accordion.Toggle>
								</Card.Header>
								<Accordion.Collapse eventKey={section.label}>
									<Card.Body>							
									{ section.items.map((component) => 	
										component.restrictionTag == 'SYS_DisableDriveA' ?
											<DriveSelection items={section.items} components={this.props.components} handleRestrictionsChange={this.props.handleRestrictionsChange}/>
										:
										component.restrictionTag.startsWith('SYS_DisableDrive') ? '' 
										:
										component.restrictionTag == 'SYS_LockPC' ?
											<LockPcRestriction lockPC={this.state.lockPC} handleRestrictionsChange={this.props.handleRestrictionsChange}/>
										:
										component.restrictionTag == 'SYS_AddFileExplorerPath' ? 
											<FileExplorerPathRestriction restrictions={this.props.restrictions} components={this.props.components} showPathTextbox={this.state.showPathTextbox} filePath={this.state.filePath} handleRestrictionsChange={this.props.handleRestrictionsChange}/>
										:						
										<InputGroup key={component.restrictionTag}>
											<Form.Check id={component.restrictionTag} key={component.restrictionTag} type="switch" defaultChecked={this.checkStatus(component.restrictionTag)} onChange={this.handleChange} label={this.addLabel(component.restrictionTag)} />
											<OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip-2">{this.addTooltip(component.restrictionTag)}</Tooltip>}>
												<svg width="2em" height="0.8em" viewBox="0 0 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
													<path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
												</svg>
											</OverlayTrigger>
										</InputGroup>
									)}
								</Card.Body>
								</Accordion.Collapse>
							</Card>
							)}
						</Accordion>
					</Container>
				</Jumbotron>
			</>
		);
	}
}

export default Desktop;