import Form from 'react-bootstrap/Form'
import Button from 'react-bootstrap/Button'
import Container from 'react-bootstrap/Container'
import Jumbotron from 'react-bootstrap/Jumbotron'
import Table from 'react-bootstrap/Table'
import OverlayTrigger from 'react-bootstrap/OverlayTrigger'
import Tooltip from 'react-bootstrap/Tooltip'
import Modal from 'react-bootstrap/Modal'
import Row from 'react-bootstrap/Row'
import Col from 'react-bootstrap/Col'
import InputGroup from 'react-bootstrap/InputGroup'
import * as yup from 'yup';
import { Formik } from 'formik';
import { MdDelete } from 'react-icons/md';


class Firewall extends React.Component {
	constructor(props) {
        super(props);
        
        let schema = yup.object({
            name: yup.string().required(),
            port: yup.number().min(1).max(65535).required(),
            action: yup.string().required(),
            protocol: yup.string().required(),
            profile: yup.string().required(),
            direction: yup.string().required(),
          });
        console.log(this.props.firewallRules) 
        this.state = {
            ruleList: this.props.firewallRules,
            rule: {},
            schema: schema,
            showWindowsRules: false
        }

        // bind the action handlers
        this.handleAdd = this.handleAdd.bind(this);
        this.handleDelete = this.handleDelete.bind(this);
        this.handleSave = this.handleSave.bind(this);
        this.handleClose = this.handleClose.bind(this);
        this.handleShow = this.handleShow.bind(this);
        this.handleErrorClose = this.handleErrorClose.bind(this);
        this.handleErrorShow = this.handleErrorShow.bind(this);   
        this.loadWindowsRules = this.loadWindowsRules.bind(this)
        this.removeWindowsRules = this.removeWindowsRules.bind(this)
        // this.handleChangeValue = this.handleChangeValue.bind(this)
    }

    handleAdd(){
        // set title
        this.setState(state => ({
            rule: {direction: '', profile: ["1","2","4"], action: '' , protocol: ''},
            showModal: true
        }))
        this.handleShow()
    }

    handleDelete(e){
        // Get rule        
        let ruleId = e.target.id    
        if (!ruleId) {
            ruleId = e.target.parentNode.id;
        }              
        // this.state.ruleList.WinFirewallRules.splice([ruleId], 1);
        // console.log(this.state.ruleList)
        // this.state.ruleList[ruleId].Operation = 'Delete';

		this.state.ruleList.map((rule, index) => {
			if (ruleId == index) {
                rule.Operation = 'Delete'
                return
			}			
		})        

        console.log(this.state.ruleList)
        this.props.handleFirewallChange(this.state.ruleList)        
		this.setState({change: ""})
    }

    handleClose() {
        this.setState({showModal: false})
    }
    
    handleShow() {           
        this.setState({showModal: true})  
    }    

    directionMapping(direction) {
        switch (direction) {
            case 1: 
                return "Inbound"  
            case 2: 
                return "Outbound"
            default:
                return "Other"
        }
    }
 
    protocolMapping(protocol) {
        switch (protocol) {
            case 6: 
                return "TCP"  
            case 17: 
                return "UDP"
            default:
                return "Any"
        }
    }

    actionMapping(action) {
        switch (action) {
            case 1: 
                return "Allow"  
            case 0: 
                return "Block"
            default:
                return "Maximum"
        }
    }

    profileMapping(profile) {
        switch (profile) {
            case 1:
                return "D"  
            case 2: 
                return "Pr"
            case 3:
                return "D-Pr"    
            case 4:
                return "Pl"
            case 5:
                return "D-Pl"
            case 6:        
                return "Pr-Pl"
            default:
                return "D-Pr-Pl"
        }
    }    

    handleSave(data) {
        
        // transform values 
        console.log(data)
        let profile = 0
        data.profile.map((value) => {
            profile += parseInt(value)
        }) 
        console.log(profile)

        // add defaults 
        let rule = { 
            "Name" : data.name, 
            "Action" : parseInt(data.action), 
            "Direction" : parseInt(data.direction), 
            "ApplicationName": null, 
            "Enabled" : true,
            "Description": "",
            "Grouping": null,
            "IcmpTypesAndCodes" : null,
            "Interfaces" : null,
            "InterfaceTypes" : "All",
            "LocalAddresses" : "*",
            "LocalPorts" : data.port,
            "Profiles" : profile,
            "Protocol" : parseInt(data.protocol),
            "RemoteAddresses" : "*",
            "RemotePorts" : "*",
            "serviceName" : null,
            "Operation": "Create"
        } 

        // this.state.ruleList.WinFirewallRules.push(rule);
        this.state.ruleList.push(rule);
        console.log(this.state.ruleList)
        
        this.props.handleFirewallChange(this.state.ruleList)
		this.setState({change: ""})        

        this.handleClose()
    }

    checkboxToInteger(checkbox) {
        if (checkbox.length > 0 || checkbox == 1) {
            return 1;
        }
        return 0;
    }
    
	checkActiveCheckbox(status) {

        let checked = false
        // console.log(status)
        if (status == 1)
            checked = true
		return checked;
	}      
    
    handleErrorClose() {
        this.setState({
            showErrorModal: false
        })
      }
    
    handleErrorShow() {
        this.setState({showErrorModal: true})  
    }

    loadWindowsRules(){
        this.setState({showWindowsRules: true})  
        console.log(this.state.showWindowsRules)
    }

    removeWindowsRules(){
        this.setState({showWindowsRules: false})  
        console.log(this.state.showWindowsRules)
    }

    render(){
        return (
            <>          
                <Jumbotron>
                    <Container>
                    
                        {/* {!this.state.showModal ? */}
                            <div style={{
                                marginBottom: '3%'
                            }}>       
                                <Row>
                                    <Col md={{ span: 5, offset: 0 }}>
                                        <h4>Lablock firewall rules
                                            <OverlayTrigger placement="right" overlay={<Tooltip id="button-windows-table-tooltip">These are Firewall rules that will be added to Windows Firewall rules table. These rules can be edited from the Lablock interface</Tooltip>}>
                                                <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                                    <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                                </svg>
                                            </OverlayTrigger>                                            
                                        </h4>
                                    </Col>
                                    <Col md={{ span: 1, offset: 6 }}>	
                                        <OverlayTrigger placement="right" overlay={<Tooltip id="button-new-rule-tooltip">Add new rule</Tooltip>}>
                                            <Button id="btn-new-rule" variant="success" onClick={this.handleAdd}>Add</Button>
                                        </OverlayTrigger>
                                    </Col>	
                                </Row>                                                       
                            </div>
                 {/*        : ''
                        } */}

                        {/* {!this.state.showModal ?                         */}

                        <Table striped bordered hover size="sm">
                        <thead>
                            <tr>
                            <th>Name 
                            <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip">Name of the firewall rule</Tooltip>}>
                                <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                    <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                </svg>
                            </OverlayTrigger>                                
                            </th>
                            <th>Port 
                            <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip">Port number of the firewall rules</Tooltip>}>
                                <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                    <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                </svg>
                            </OverlayTrigger>                                  
                            </th>
                            <th>Action 
                            <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip">Allows or Blocks the firewall rule connection</Tooltip>}>
                            <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                    <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                </svg>
                            </OverlayTrigger>                                  
                            </th>
                            <th>Direction 
                            <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip">Inbound or Outbound firewall rule</Tooltip>}>
                                <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                    <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                </svg>
                            </OverlayTrigger>  
                            </th>
                            <th>Protocol 
                            <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip">Protocol of the firewall rule</Tooltip>}>
                                <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                    <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                </svg>
                            </OverlayTrigger>  
                            </th>
                            <th>Interface 
                            <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip">Network location of the rule. Can be Domain, Public and Private. Multiple options are allowed</Tooltip>}>
                                <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                    <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                </svg>
                            </OverlayTrigger>                                  
                            </th>
                            <th></th>
                            </tr>
                        </thead>   
                        <tbody>
                            {this.state.ruleList ?
                            this.state.ruleList.map((rule, index) =>
                            rule.Operation != "Delete" ?
                            <tr key={index}>
                                <td>{rule.Name}</td>
                                <td>{rule.LocalPorts}</td>
                                <td>{this.actionMapping(rule.Action)}</td>
                                <td>{this.directionMapping(rule.Direction)}</td>
                                <td>{this.protocolMapping(rule.Protocol)}</td>
                                <td>{this.profileMapping(rule.Profiles)}</td>
                                <td>
                                    <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip-delete">Delete</Tooltip>}>
                                        {/* <Button id={index} variant="variant-dark" onClick={this.handleDelete}> */}
                                        <Button id={index} variant="variant-dark" onClick={this.handleDelete}>
                                            <svg id={index} width="1em" height="1em" viewBox="0 0 16 16" className="bi bi-x-circle" fill="currentColor" xmlns="http://www.w3.org/2000/svg">
                                                <path fillRule="evenodd" d="M8 15A7 7 0 1 0 8 1a7 7 0 0 0 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/>
                                                <path fillRule="evenodd" d="M4.646 4.646a.5.5 0 0 1 .708 0L8 7.293l2.646-2.647a.5.5 0 0 1 .708.708L8.707 8l2.647 2.646a.5.5 0 0 1-.708.708L8 8.707l-2.646 2.647a.5.5 0 0 1-.708-.708L7.293 8 4.646 5.354a.5.5 0 0 1 0-.708z"/>
                                            </svg>                                            
                                        </Button>
                                    </OverlayTrigger>
                                </td>
                            </tr>
                            : '' 
                            )
                            : ''
                            }
                        </tbody>         
                        </Table>
                        {/*  end listing */}
                        
                        <Row>
                            <Col md={{ span: 5, offset: 0 }}>
                                <h4>Windows firewall rules
                                <OverlayTrigger placement="right" overlay={<Tooltip id="button-windows-table-tooltip">These rules are Firewall rules from Windows for visibility. These rules cannot be deleted from Windows using the Lablock interface</Tooltip>}>
                                    <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                        <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                    </svg>
                                </OverlayTrigger>
                                </h4>
                            </Col>
                            <Col md={{ span: 3, offset: 4 }}>
                                {!this.state.showWindowsRules ?
                                <Button id="btn-load-rule" variant="info" onClick={this.loadWindowsRules}>Load default Windows rules</Button>
                                :
                                <Button id="btn-load-rule" variant="info" onClick={this.removeWindowsRules}>Clear default Windows rules</Button>
                                }
                            </Col>
                        </Row>  

                        <Table striped bordered hover size="sm">
                        <thead>
                            <tr>
                            <th>Name 
                            <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip">Name of the firewall rule</Tooltip>}>
                                <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                    <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                </svg>
                            </OverlayTrigger>                                
                            </th>
                            <th>Port 
                            <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip">Port number of the firewall rules</Tooltip>}>
                                <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                    <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                </svg>
                            </OverlayTrigger>                                  
                            </th>
                            <th>Action 
                            <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip">Allows or Blocks the firewall rule connection</Tooltip>}>
                            <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                    <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                </svg>
                            </OverlayTrigger>                                  
                            </th>
                            <th>Direction 
                            <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip">Inbound or Outbound firewall rule</Tooltip>}>
                                <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                    <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                </svg>
                            </OverlayTrigger>  
                            </th>
                            <th>Protocol 
                            <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip">Protocol of the firewall rule</Tooltip>}>
                                <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                    <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                </svg>
                            </OverlayTrigger>  
                            </th>
                            <th>Interface 
                            <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip">Network location of the rule. Can be Domain, Public and Private. Multiple options are allowed</Tooltip>}>
                                <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                    <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                </svg>
                            </OverlayTrigger>                                  
                            </th>
                            <th></th>
                            </tr>
                        </thead>   
                        <tbody>
                            {this.state.showWindowsRules ?
                            this.props.windowsRules.map((rule, index) =>                           
                            <tr key={index}>
                                <td>{rule.Name}</td>
                                <td>{rule.LocalPorts}</td>
                                <td>{this.actionMapping(rule.Action)}</td>
                                <td>{this.directionMapping(rule.Direction)}</td>
                                <td>{this.protocolMapping(rule.Protocol)}</td>
                                <td>{this.profileMapping(rule.Profiles)}</td>
                                <td></td>
                            </tr>
                            )
                            : ''
                            }
                        </tbody>         

                        </Table>

                        {/* // start form   */}
                        <Modal show={this.state.showModal} onHide={this.handleClose} aria-labelledby="contained-modal-title-vcenter" centered>
                            <Modal.Header closeButton>
                                <Modal.Title>Add new firewall rule</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>                            
 
                                <Formik
                                    enableReinitialize={true}
                                    validationSchema={this.state.schema}
                                    onSubmit={(values) => {
                                    console.log(values)
                                    this.handleSave(values);
                                    }}
                                    initialValues={this.state.rule}
                                >
                                {({
                                handleSubmit,
                                handleChange,
                                handleBlur,
                                values,
                                touched,
                                isValid,
                                errors,
                                }) => (
                                <Form onSubmit={handleSubmit}>                        
                                    <Form.Group as={Row} controlId="name">
                                        <Form.Label column sm="2">Name</Form.Label>
                                        <Col sm="5">
                                            <Form.Control 
                                                type="text"
                                                name="name"
                                                value={values.name}
                                                onChange={handleChange}
                                                onBlur={handleBlur}
                                                isInvalid={!!errors.name}
                                            />
                                            <Form.Control.Feedback type="invalid">{errors.name}</Form.Control.Feedback>                                    
                                        </Col>
                                    </Form.Group>

                                    <Form.Group as={Row} controlId="port">
                                        <Form.Label column sm="2">Port</Form.Label>
                                        <Col sm="5">
                                            <Form.Control 
                                                type="text"
                                                name="port"
                                                value={values.port}
                                                onChange={handleChange}
                                                onBlur={handleBlur}
                                                isInvalid={!!errors.port}
                                            />
                                        </Col>
                                    </Form.Group> 

                                    <Form.Group as={Row} >
                                        <Form.Label column sm={2}>Direction</Form.Label>
                                        <Col sm="8">
                                            <Form.Check custom inline label="Inbound" type="radio" name="direction" id="radio-inbound" value="1" onChange={handleChange} isInvalid={!!errors.direction}/>
                                            <Form.Check custom inline label="Outbound" type="radio" name="direction" id="radio-outbound" value="2" onChange={handleChange} isInvalid={!!errors.direction}/>
                                        </Col>
                                    </Form.Group>

                                    <Form.Group as={Row} >
                                        <Form.Label column sm={2}>Protocol</Form.Label>
                                        <Col sm="8">
                                            <Form.Check custom inline label="TCP" type="radio" name="protocol" id="radio-tcp" value="6" onChange={handleChange} isInvalid={!!errors.protocol}/>
                                            <Form.Check custom inline label="UDP" type="radio" name="protocol" id="radio-udp" value="17" onChange={handleChange} isInvalid={!!errors.protocol}/>
                                        </Col>
                                    </Form.Group>

                                    <Form.Group as={Row} >
                                        <Form.Label column sm={2}>Action</Form.Label>
                                        <Col sm="8">
                                            <Form.Check custom inline label="Allow" type="radio" name="action" id="radio-allow" value="1" onChange={handleChange} isInvalid={!!errors.action}/>
                                            <Form.Check custom inline label="Block" type="radio" name="action" id="radio-block" value="0" onChange={handleChange} isInvalid={!!errors.action}/>
                                        </Col>
                                    </Form.Group> 

                                    <Form.Group as={Row} >
                                        <Form.Label column sm={2}>Profile</Form.Label>
                                        <Col sm="8">
                                            <Form.Check custom inline label="Public" type="checkbox" name="profile" id="radio-public" value="4" onChange={handleChange} defaultChecked="true" isInvalid={!!errors.profile}/>
                                            <Form.Check custom inline label="Private" type="checkbox" name="profile" id="radio-private" value="2" onChange={handleChange} defaultChecked="true" isInvalid={!!errors.profile}/>
                                            <Form.Check custom inline label="Domain" type="checkbox" name="profile" id="radio-domain" value="1" onChange={handleChange} defaultChecked="true" isInvalid={!!errors.profile}/>
                                        </Col>
                                    </Form.Group>  
                                    <Col sm="20">
                                        <Button variant="danger"  type="submit">Save</Button>{' '}
                                        <Button variant="secondary" onClick={this.handleClose}>Close</Button>
                                    </Col>                          
                                </Form>                                
                            )}
                            </Formik>
                        </Modal.Body>  
{/* 
                        <Modal.Footer>
                            <Button variant="danger" onClick={this.handleSave}>Save</Button>
                            <Button variant="secondary" onClick={this.handleClose}>Close</Button>
                        </Modal.Footer>                         */}
                    </Modal>
                    

                        <Modal show={this.state.showErrorModal} onHide={this.handleErrorClose} aria-labelledby="contained-modal-title-vcenter" centered>
                            <Modal.Header closeButton>
                                <Modal.Title>Ops, something didn't work as expected</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>There has been an error when processing your request, please try again or contact support</Modal.Body>
                            <Modal.Footer>
                                <Button variant="secondary" onClick={this.handleErrorClose}>Close</Button>
                            </Modal.Footer>
                        </Modal>                          
                    </Container>
                </Jumbotron>
            </>
        );
  } 
}

export default Firewall;