import Button from 'react-bootstrap/Button'
import Container from 'react-bootstrap/Container'
import Jumbotron from 'react-bootstrap/Jumbotron'
import Form from 'react-bootstrap/Form'
import FormControl from 'react-bootstrap/FormControl'
import Row from 'react-bootstrap/Row'
import Col from 'react-bootstrap/Col'
import InputGroup from 'react-bootstrap/InputGroup'
import Modal from 'react-bootstrap/Modal'
import * as yup from 'yup';
import { Formik } from 'formik';
import log from '../../lib/log'
import AES from '../../lib/aes'


class General extends React.Component {
	constructor(props) {
        super(props);

        console.log(this.props)

        let schema = yup.object({
            lablockPort: yup.number().min(0).max(65535).required(),
            sessionExpiration: yup.number().min(5).max(60).required(),
            suspendRestrictionsTime: yup.number().min(1).max(480).required(),
            suspendRestrictionsHotkey: yup.string(),
            hotkeyPassword: yup.string().required(),
            hideUserNotifications: yup.bool(),
            brand: yup.string().max(30),
            windowsBackgroundImage: yup.string().matches(/^(.+)\\([^/]+)$/),
            snapshotsPath: yup.string().matches(/^(.+)\\([^/]+)$/).required(),            
            backupRetentionPeriod: yup.number().min(1).max(30).required(),
            backupLocation: yup.string().matches(/^(.+)\\([^/]+)$/)
          });

        this.state = {
            showApplyModal: false,
            showErrorModal: false,
            schema: schema,
        };

        if (this.props.generalSettings == null) {
            this.props.generalSettings = {}
        }

        // bind the action handlers
        this.handleApplyClose = this.handleApplyClose.bind(this);
        this.handleApplyShow = this.handleApplyShow.bind(this);  
        this.handleApplyGeneralSettings = this.handleApplyGeneralSettings.bind(this);

        this.handleErrorClose = this.handleErrorClose.bind(this);
        this.handleErrorShow = this.handleErrorShow.bind(this);    
      
    }

	componentDidMount(){
		var getUrl = window.location;		
		console.log(getUrl.host)
		var hostIP = getUrl.host
		this.setState(state => ({host: hostIP}))
    }

    handleApplyClose() {
        this.setState({showApplyModal: false})
    }

    handleApplyShow() {
        this.setState({showApplyModal: true})
    }

    handleApplyGeneralSettings(data) {
        
        console.log(data)
        console.log(this.props.generalSettings)

        let payload = {
            'lablockPort': data.lablockPort, 
            'sessionExpiration': data.sessionExpiration, 
            'suspendRestrictionsTime': data.suspendRestrictionsTime, 
            'suspendRestrictionsHotkey': data.suspendRestrictionsHotkey,
            'hotkeyPassword': data.hotkeyPassword, 
            'brand': data.brand,
            'hideUserNotifications': data.hideUserNotifications, 
            'windowsBackgroundImage': data.windowsBackgroundImage, 
            'windowsBackgroundImageStyle': 6,
            'snapshotsPath': data.snapshotsPath,
            'backupLocation': data.backupLocation,
            'backupRetentionPeriod': data.backupRetentionPeriod
        };

        const applyLablockSettingsEndpoint = this.props.serviceURL + '/lablocksettings'
        console.log(applyLablockSettingsEndpoint)
  
        payload = AES.Encrypt(JSON.stringify(payload), this.props.JAKO, this.props.OJAK)
        fetch(applyLablockSettingsEndpoint, {method: 'POST', body: payload})
        .then(response => {
             this.handleApplyShow()
            // log outcome

            if (data.lablockPort != this.props.generalSettings.lablockPort) 
                log(this.props.user.username, this.props.logModule, "changed lablock port from " + this.props.generalSettings.lablockPort + " to " + data.lablockPort + " from ", this.state.host)
            
            if (data.sessionExpiration != this.props.generalSettings.sessionExpiration)
                log(this.props.user.username, this.props.logModule, "changed session expiration time from " + this.props.generalSettings.sessionExpiration + " minutes to " + data.sessionExpiration + " minutes from ", this.state.host)

            if (data.suspendRestrictionsTime != this.props.generalSettings.suspendRestrictionsTime)
                log(this.props.user.username, this.props.logModule, "changed suspend restrictions time from " + this.props.generalSettings.suspendRestrictionsTime + " minutes to " + data.suspendRestrictionsTime + " minutes from ", this.state.host)

            if (data.suspendRestrictionsHotkey != this.props.generalSettings.suspendRestrictionsHotkey)
                log(this.props.user.username, this.props.logModule, " changed hotkey combination from " + this.props.generalSettings.suspendRestrictionsHotkey + " to " + data.suspendRestrictionsHotkey + " from ", this.state.host)

            if (data.hotkeyPassword != this.props.generalSettings.hotkeyPassword)
                log(this.props.user.username, this.props.logModule, " changed hotkey password from ", this.state.host)

            if (data.hideUserNotifications != this.props.generalSettings.hideUserNotifications) {
                if (data.hideUserNotifications == true)
                    log(this.props.user.username, this.props.logModule, " activated hide user notifications from " , this.state.host)
                else
                    log(this.props.user.username, this.props.logModule, " disabled hide user notifications from " , this.state.host)
            }

            if (data.brand != this.props.generalSettings.brand) {
                if (data.brand == "")
                    log(this.props.user.username, this.props.logModule, " removed brand from ", this.state.host)
                else if (this.props.generalSettings.brand == "")    
                    log(this.props.user.username, this.props.logModule, " added " + data.brand + " brand from ", this.state.host)
                else     
                    log(this.props.user.username, this.props.logModule, " changed " + data.brand + " brand from ", this.state.host)
            }
            if (data.windowsBackgroundImage != this.props.generalSettings.windowsBackgroundImage) {
                if (data.windowsBackgroundImage == "")
                    log(this.props.user.username, this.props.logModule, " removed background image from ", this.state.host)
                else if (this.props.generalSettings.brand == "")    
                    log(this.props.user.username, this.props.logModule, " added " + data.windowsBackgroundImage + " as background image ", this.state.host)
                else     
                    log(this.props.user.username, this.props.logModule, " changed background image from " + this.props.generalSettings.windowsBackgroundImage + " to " + data.windowsBackgroundImage + " from ", this.state.host)
            }
            if (data.snapshotsPath != this.props.generalSettings.snapshotsPath)
                log(this.props.user.username, this.props.logModule, " changed snapshot path from " + this.props.generalSettings.snapshotsPath + " to " + data.snapshotsPath + " from ", this.state.host)

            if (data.backupRetentionPeriod != this.props.generalSettings.backupRetentionPeriod)
                log(this.props.user.username, this.props.logModule, "changed backup retention period from " + this.props.generalSettings.backupRetentionPeriod + " days to " + data.backupRetentionPeriod + " days from ", this.state.host)
            if (data.backupLocation != this.props.generalSettings.backupLocation)
                log(this.props.user.username, this.props.logModule, " changed backups path from " + this.props.generalSettings.backupLocation + " to " + data.backupLocation + " from ", this.state.host)
    

        })       
        .catch(error => {
            this.handleErrorShow()
            console.error('There has been a problem with your fetch operation:', error);
        });   
    }
    
    handleErrorClose() {
        this.setState({showErrorModal: false})
    }

    handleErrorShow() {
        this.setState({showErrorModal: true})  
    }    

    render(){
        return (
            <>     
                <Jumbotron>
                    <Container>
                        <Formik
                                validationSchema={this.state.schema}
                                onSubmit={(values) => {
                                    console.log(values)
                                    this.handleApplyGeneralSettings(values);
                                }}
                                initialValues={this.props.generalSettings}
                            >
                            {({
                            handleSubmit,
                            handleChange,
                            handleBlur,
                            values,
                            touched,
                            isValid,
                            errors,
                            }) => (
                            <Form onSubmit={handleSubmit}>                         
                                <Form.Group as={Row} controlId="lablockPort">
                                    <Form.Label column sm="4">Lablock port</Form.Label>
                                    <Col sm="3">
                                        <Form.Control 
                                            name="lablockPort"
                                            value={values.lablockPort}
                                            onChange={handleChange} 
                                            onBlur={handleBlur}
                                            isInvalid={!!errors.lablockPort && touched.lablockPort}
                                        />
                                        <Form.Control.Feedback type="invalid">{errors.lablockPort}</Form.Control.Feedback>
                                    </Col>
                                </Form.Group>

                                <Form.Group as={Row} controlId="sessionExpiration">
                                    <Form.Label column sm="4">Session expiration</Form.Label>
                                    <Col sm="3">
                                        <InputGroup className="mb-2">
                                            <FormControl 
                                                name="sessionExpiration" 
                                                value={values.sessionExpiration}
                                                onChange={handleChange} 
                                                onBlur={handleBlur}
                                                isInvalid={!!errors.sessionExpiration && touched.sessionExpiration}
                                            />
                                            <Form.Control.Feedback type="invalid">{errors.sessionExpiration && touched.sessionExpiration}</Form.Control.Feedback>
                                            <InputGroup.Prepend>
                                                <InputGroup.Text>Minutes</InputGroup.Text>
                                            </InputGroup.Prepend>
                                        </InputGroup>                                
                                    </Col>
                                </Form.Group> 

                                <Form.Group as={Row} controlId="suspensionTime">
                                    <Form.Label column sm="4">Suspend restrictions time</Form.Label>
                                    <Col sm="3">
                                        <InputGroup className="mb-2">
                                            <FormControl 
                                                name="suspendRestrictionsTime" 
                                                value={values.suspendRestrictionsTime}
                                                onChange={handleChange}  
                                                onBlur={handleBlur}
                                                isInvalid={!!errors.suspendRestrictionsTime && touched.suspendRestrictionsTime}                                             
                                            />
                                            <Form.Control.Feedback type="invalid">{errors.suspendRestrictionsTime}</Form.Control.Feedback>
                                            <InputGroup.Prepend>
                                                <InputGroup.Text>Minutes</InputGroup.Text>
                                            </InputGroup.Prepend>
                                        </InputGroup>                                
                                    </Col>
                                </Form.Group> 

                                <Form.Group as={Row} controlId="suspendRestrictionsHotkey">
                                    <Form.Label column sm="4">Suspend restrictions Hotkey combination</Form.Label>
                                    <Col sm="3">
                                        <Form.Control as="select" defaultValue={values.suspendRestrictionsHotkey} onChange={handleChange}  >
                                            <option value="LControlKey+LWin+D7">Control + Winkey + 7</option>
                                            <option value="LControlKey+LWin+D9">Control + Winkey + 9</option>
                                            <option value="LControlKey+LWin+A">Control + Winkey + A</option>
                                            <option value="LControlKey+LWin+Z">Control + Winkey + Z</option>
                                            <option value="LControlKey+LWin+L">Control + Winkey + L</option>
                                            <option value="LControlKey+ShiftKey+D7">Control + Shift + 7</option>
                                            <option value="LControlKey+ShiftKey+D9">Control + Shift + 9</option>
                                            <option value="LControlKey+ShiftKey+A">Control + Shift + A</option>
                                            <option value="LControlKey+ShiftKey+Z">Control + Shift + Z</option>
                                            <option value="LControlKey+ShiftKey+L">Control + Shift + L</option>
                                        </Form.Control>
                                    </Col>
                                </Form.Group> 

                                <Form.Group as={Row} controlId="hotkeyPassword">
                                    <Form.Label column sm="4">Hotkey password</Form.Label>
                                    <Col sm="3">
                                        <Form.Control 
                                            type="password" 
                                            value={values.hotkeyPassword}
                                            onChange={handleChange}  
                                            onBlur={handleBlur}
                                            isInvalid={!!errors.hotkeyPassword && touched.hotkeyPassword}                                               
                                        />
                                        <Form.Control.Feedback type="invalid">{errors.hotkeyPassword}</Form.Control.Feedback>
                                    </Col>
                                </Form.Group>

                                <Form.Group as={Row} controlId="hideUserNotifications">
                                    <Form.Label column sm="4">Hide user notifications</Form.Label>
                                    <Col sm={{ span: 0, offset: 0 }}>
                                        <Form.Check 
                                            type="switch" 
                                            label="" 
                                            onChange={handleChange}
                                            defaultChecked={this.props.generalSettings.hideUserNotifications} 
                                        />
                                    </Col>
                                </Form.Group> 

                                <Form.Group as={Row} controlId="brand">
                                    <Form.Label column sm="4">Brand</Form.Label>
                                    <Col sm="3">
                                        <Form.Control 
                                            name="brand"
                                            value={values.brand}
                                            onChange={handleChange}  
                                            onBlur={handleBlur}
                                            isInvalid={!!errors.brand && touched.brand}                                               
                                        />
                                        <Form.Control.Feedback type="invalid">{errors.brand}</Form.Control.Feedback>
                                    </Col>
                                </Form.Group>

                                <Form.Group as={Row} controlId="windowsBackgroundImage">
                                    <Form.Label column sm="4">Choose Windows Background image</Form.Label>
                                    <Col sm="5">
                                        <Form.Control 
                                            name = "windowsBackgroundImage"
                                            value={values.windowsBackgroundImage}
                                            onChange={handleChange}  
                                            onBlur={handleBlur}
                                            isInvalid={!!errors.windowsBackgroundImage && touched.windowsBackgroundImage}                                               
                                        />                                        
                                    </Col>
                                </Form.Group> 

                                <Form.Group as={Row} controlId="snapshotsPath">
                                    <Form.Label column sm="4">Snapshots path</Form.Label>
                                    <Col sm="5">
                                        <Form.Control 
                                            name = "snapshotsPath"
                                            value={values.snapshotsPath}
                                            onChange={handleChange}  
                                            onBlur={handleBlur}
                                            isInvalid={!!errors.snapshotsPath && touched.snapshotsPath}                                               
                                        />
                                        <Form.Control.Feedback type="invalid">{errors.snapshotsPath}</Form.Control.Feedback>
                                    </Col>
                                </Form.Group> 

                                <Form.Group as={Row} controlId="backupRetentionPeriod">
                                    <Form.Label column sm="4">Backups retention period</Form.Label>
                                    <Col sm="3">
                                        <InputGroup className="mb-2">
                                            <FormControl 
                                                name="backupRetentionPeriod" 
                                                value={values.backupRetentionPeriod}
                                                onChange={handleChange} 
                                                onBlur={handleBlur}
                                                isInvalid={!!errors.backupRetentionPeriod && touched.backupRetentionPeriod}
                                            />
                                            <Form.Control.Feedback type="invalid">{errors.backupRetentionPeriod && touched.backupRetentionPeriod}</Form.Control.Feedback>
                                            <InputGroup.Prepend>
                                                <InputGroup.Text>Days</InputGroup.Text>
                                            </InputGroup.Prepend>
                                        </InputGroup>                                
                                    </Col>
                                </Form.Group> 

                                <Form.Group as={Row} controlId="backupLocation">
                                    <Form.Label column sm="4">Backups path</Form.Label>
                                    <Col sm="5">
                                        <Form.Control 
                                            name="backupLocation"
                                            value={values.backupLocation}
                                            onChange={handleChange}  
                                            onBlur={handleBlur}
                                            isInvalid={!!errors.backupLocation && touched.backupLocation}                                               
                                        />
                                        <Form.Control.Feedback type="invalid">{errors.backupLocation}</Form.Control.Feedback>
                                    </Col>
                                </Form.Group> 
                                <Col md={{ span: 1, offset: 4 }}>
                                    <Button id="btn-apply-restrictions" type="submit" variant="dark" >
                                        Apply
                                    </Button>	
                                </Col>	
                            </Form>
                        )}
                        </Formik>                            



                        <Modal show={this.state.showApplyModal} onHide={this.handleApplyClose} aria-labelledby="contained-modal-title-vcenter" centered>
                            <Modal.Header closeButton>
                                <Modal.Title>General settings</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>Application settings applied successfully</Modal.Body>
                            <Modal.Footer>
                                <Button id="btn-cancel-generalsettings" variant="secondary" onClick={this.handleApplyClose}>
                                Close
                                </Button>
                            </Modal.Footer>
                        </Modal>

                        <Modal show={this.state.showErrorModal} onHide={this.handleErrorClose} aria-labelledby="contained-modal-title-vcenter" centered>
                            <Modal.Header>
                                <Modal.Title>Ops, something didn't work as expected</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>There has been an error when processing your request, please try again or contact support</Modal.Body>
                            <Modal.Footer>
                                <Button variant="secondary" onClick={this.handleErrorClose}>
                                Close
                                </Button>
                            </Modal.Footer>
                        </Modal>                         
                    </Container>
                </Jumbotron>
            </>
        );
  } 
}

export default General;