import Head from 'next/head';
import Form from 'react-bootstrap/Form'
import {Navbar, Nav, NavDropdown } from 'react-bootstrap';
import { withRouter } from 'next/router'
import AES from '../../lib/aes'
import Modal from 'react-bootstrap/Modal'
import Button from 'react-bootstrap/Button'
import Table from 'react-bootstrap/Table'
import { RiAccountCircleFill } from "react-icons/ri";

class Header extends React.Component {

    constructor(props) {
		super(props);

		// console.log(this.props.permissions.permissions)
		if (this.props.permissions != null){
			this.state = {
				host: '',
				licenseDetails: {},
				permissions: this.props.permissions.permissions,
				user: this.props.user
			}
		} else {
			this.state = {
				permissions: [],
				user: {},
				licenseDetails: {}
			}
		}

		this.logout = this.logout.bind(this);
		this.handleLicenseModalClose = this.handleLicenseModalClose.bind(this)
		this.handleLicenseModal = this.handleLicenseModal.bind(this)
	}

	componentDidMount(){
		var getUrl = window.location;		
		console.log(getUrl.host)
		var hostIP = getUrl.host
		this.setState(state => ({host: hostIP}))

		fetch(this.props.serviceURL + '/computerdetails')
		.then(response => response.text())
		.then(response => {
			let computerDetails = JSON.parse(AES.Decrypt(response, this.props.JAKO, this.props.OJAK))
			this.setState(state => ({version: computerDetails.LabLockVersion, computerName: computerDetails.computerName}))
		})

		fetch(this.props.serviceURL + '/licenseinfo')
		.then(response => response.text())
		.then(response => {
			let licenseDetails = JSON.parse(AES.Decrypt(response, this.props.JAKO, this.props.OJAK))
			this.setState(state => ({licenseDetails: licenseDetails}))
		})		

		// keep session alive
		let payloadSession = {
			'user': this.props.user,
			'permissions':this.props.permissions.permissions,
		}
		fetch('/api/sessions', {method: 'POST', headers: {'Accept': 'application/json', 'Content-Type': 'application/json'}, body: JSON.stringify(payloadSession)})
		.catch(err => console.log(err));		
	}

	logout(){
		fetch('/api/logout', {method: 'POST'})
		.then(res => {
				if (res.status == 200) {
					// route to login
					window.location.assign("http://" + window.location.host)
				}	
			}

		)
		.catch(err => console.log(err));

		// this.props.router.push('/');
	}

    handleLicenseModalClose() {
        this.setState(state => ({showLicenseModal: false}))
    }

    handleLicenseModal() {
		this.setState(state => ({showLicenseModal: true}))
	}  
		
    render() {      
        return (
            <>   
				<style type="text/css">
					{`
					.dropdown-menu.show {
						right: 0;
						left: -300%;
					}

					`}
				</style>			
				<div style={{
					marginBottom: '5%'
				}}>
					<Head>
						<title>LabLock</title>
						<link rel="icon" href="/favicon.ico" />
					</Head>	
					<Navbar fixed="top" collapseOnSelect expands="lg" bg="dark" variant="dark">
						<Navbar.Brand href="/dashboard">
							<img
								src="/logo.jpeg"
								width="35"
								height="35"
								className="d-inline-block align-top"	
								alt="JAKO-solutions"							
							/>
						</Navbar.Brand>
						<Nav className="mr-auto">
							{this.state.permissions.indexOf("dashboard") >= 0 ?
							<Nav.Link href="/dashboard">Dashboard</Nav.Link>
							: ''}
							{this.state.permissions.indexOf("restrictions") >= 0 ?
							<Nav.Link href="/restrictions">Restrictions</Nav.Link>
							: ''}
							{this.state.permissions.indexOf("activity") >= 0 ?
							<Nav.Link href="/activity">Activity</Nav.Link>
							: ''}
							{this.state.permissions.indexOf("settings") >= 0 ?
							<Nav.Link href="/settings">Settings</Nav.Link>
							: ''}
							{this.state.permissions.indexOf("import_export") >= 0 ?
							<Nav.Link href="/importexport">Import/Export</Nav.Link>
							: ''}
						</Nav>
						<Nav className="ml-auto pr-md-5"> 
							<Form inline>
								<RiAccountCircleFill color="grey" size="30px"/>

								<NavDropdown title={this.state.user.fullname} id="logout-menu-dropdown" >
									<NavDropdown.Item >User: {this.state.user.username}</NavDropdown.Item>
									<NavDropdown.Item >Role: {this.state.user.role}</NavDropdown.Item>
									<NavDropdown.Item >IP: {this.state.host} </NavDropdown.Item>
									<NavDropdown.Item >Lablock version: {this.state.version}</NavDropdown.Item>
									<NavDropdown.Item onClick={this.handleLicenseModal}>License status</NavDropdown.Item>
									<NavDropdown.Divider />
									<NavDropdown.Item href="http://jako-solutions.com/documentation">
										<svg width="1em" height="1em" viewBox="0 0 16 16" className="bi bi-question-circle" fill="currentColor" xmlns="http://www.w3.org/2000/svg">
											<path fillRule="evenodd" d="M8 15A7 7 0 1 0 8 1a7 7 0 0 0 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/>
											<path d="M5.255 5.786a.237.237 0 0 0 .241.247h.825c.138 0 .248-.113.266-.25.09-.656.54-1.134 1.342-1.134.686 0 1.314.343 1.314 1.168 0 .635-.374.927-.965 1.371-.673.489-1.206 1.06-1.168 1.987l.003.217a.25.25 0 0 0 .25.246h.811a.25.25 0 0 0 .25-.25v-.105c0-.718.273-.927 1.01-1.486.609-.463 1.244-.977 1.244-2.056 0-1.511-1.276-2.241-2.673-2.241-1.267 0-2.655.59-2.75 2.286zm1.557 5.763c0 .533.425.927 1.01.927.609 0 1.028-.394 1.028-.927 0-.552-.42-.94-1.029-.94-.584 0-1.009.388-1.009.94z"/>
										</svg>   Help										
									</NavDropdown.Item>
									<NavDropdown.Item onClick={this.logout}>
										<svg width="1em" height="1em" viewBox="0 0 16 16" className="bi bi-box-arrow-right" fill="currentColor" xmlns="http://www.w3.org/2000/svg">
											<path fillRule="evenodd" d="M10 12.5a.5.5 0 0 1-.5.5h-8a.5.5 0 0 1-.5-.5v-9a.5.5 0 0 1 .5-.5h8a.5.5 0 0 1 .5.5v2a.5.5 0 0 0 1 0v-2A1.5 1.5 0 0 0 9.5 2h-8A1.5 1.5 0 0 0 0 3.5v9A1.5 1.5 0 0 0 1.5 14h8a1.5 1.5 0 0 0 1.5-1.5v-2a.5.5 0 0 0-1 0v2z"/>
											<path fillRule="evenodd" d="M15.854 8.354a.5.5 0 0 0 0-.708l-3-3a.5.5 0 0 0-.708.708L14.293 7.5H5.5a.5.5 0 0 0 0 1h8.793l-2.147 2.146a.5.5 0 0 0 .708.708l3-3z"/>
										</svg>	 Logout
									</NavDropdown.Item>
								</NavDropdown>								
							</Form>
						</Nav>				
					</Navbar>		
				</div>		
				<Modal show={this.state.showLicenseModal} onHide={this.handleLicenseModalClose} size="lg" aria-labelledby="contained-modal-title-vcenter" centered>
					<Modal.Header>
						<Modal.Title>License details</Modal.Title>
					</Modal.Header>
					<Modal.Body>
						<Table striped bordered hover>
							<tbody>
								<tr>
									<th>License type</th>
									<th>{this.state.licenseDetails.IsPerpetual == false ? "Temporary" :  "Permanent"} </th>
								</tr>
								{this.state.licenseDetails.IsPerpetual == false ?
								<tr>
									<th>Expiration date</th>
									<th>{new Date(this.state.licenseDetails.ValidityDateTime).toLocaleDateString()}</th>
								</tr>					
								:  ''}
								<tr>
									<th>License ID</th>
									<th>{this.state.licenseDetails.LicenseID} </th>
								</tr>
								<tr>
									<th>Computer</th>
									<th>{this.state.computerName}  </th>
								</tr>
								<tr>
									<th>Customer name</th>
									<th>{this.state.licenseDetails.CustomerName}</th>
								</tr>
								<tr>
									<th>Customer ID</th>
									<th>{this.state.licenseDetails.CustomerID}</th>
								</tr>

							</tbody>         
						</Table>						
					</Modal.Body>
					<Modal.Footer>
						<Button variant="secondary" onClick={this.handleLicenseModalClose}>
						Close
						</Button>
					</Modal.Footer>
				</Modal> 				
			</>
        );
  } 
}

export default withRouter(Header);