import Button from 'react-bootstrap/Button'
import Container from 'react-bootstrap/Container'
import Jumbotron from 'react-bootstrap/Jumbotron'
import Form from 'react-bootstrap/Form'
import Row from 'react-bootstrap/Row'
import Col from 'react-bootstrap/Col'
import Modal from 'react-bootstrap/Modal'
import OverlayTrigger from 'react-bootstrap/OverlayTrigger'
import Tooltip from 'react-bootstrap/Tooltip'
import { Formik } from 'formik';
import { saveAs } from 'file-saver';
import AES from '../../lib/aes'
import log from '../../lib/log'
import bsCustomFileInput from 'bs-custom-file-input';

class ImportExportContainer extends React.Component {
	constructor(props) {
		super(props);

        let initialValues = {	
            "ksettings" : true,
            "ksyskeys" : true,
            "kstartup" : true,
            "kappblocker" : true,
            "kfirewall" : true,
            "LDAPSettings" : true,
            "LabLockSettings" : true,
            "lablockData" : true
        
        }
        this.state = {
            showErrorModal: false,
            showModal: false,
            changePath: '',
            initialValues: initialValues
        };

        // bind the action handlers
        this.export = this.export.bind(this);
        // this.changePath = this.changePath.bind(this);
        this.handleClose = this.handleClose.bind(this);
        this.handleImportSuccessShow = this.handleImportSuccessShow.bind(this);
        this.handleExportSuccessShow = this.handleExportSuccessShow.bind(this);
        this.handleErrorShow = this.handleErrorShow.bind(this);     
        this.handleFileSelected = this.handleFileSelected.bind(this); 
        this.handleImport = this.handleImport.bind(this);  
    }

	componentDidMount(){
        bsCustomFileInput.init()    

		var getUrl = window.location;		
		console.log(getUrl.host)
		var hostIP = getUrl.host
        this.setState(state => ({host: hostIP}))
        
        const computerDetailsEndpoint = this.props.serviceURL + '/computerdetails'
        fetch(computerDetailsEndpoint)
        .then(response => response.text())
        .then(response => {
            let responseDecoded = JSON.parse(AES.Decrypt(response, this.props.JAKO, this.props.OJAK))
            this.setState(state => ({computerName: responseDecoded.computerName}))
        })    
    }
        
    export(){
        const exportEndpoint = this.props.serviceURL + '/exportlocal'
        fetch(exportEndpoint)      
        .then(response => response.text())
        .then(response => {
            // console.log(response)
            let filename = this.createExportFilename()
            let responseDecoded = JSON.parse(AES.Decrypt(response, this.props.JAKO, this.props.OJAK))
            var blob = new Blob([responseDecoded.ExportFile.Base64String], {type: "text/plain;charset=utf-8"});
            saveAs(blob, filename);                
            this.handleExportSuccessShow()
            // log outcome
            log(this.props.userObj.username, this.props.logModule, "exported the configuration file with the identification number MD5 "+ responseDecoded.ExportFile.ExportFileMD5 +" from ", this.state.host)
        })      
        .catch(error => {
            this.handleErrorShow()
            console.error('There has been a problem with your fetch operation:', error);
        });    
    }

    createExportFilename() {
        const dateTime = new Date()
        let date = dateTime.toLocaleDateString().split('/')
        let time = dateTime.toLocaleTimeString().split(':')
        let filename = date[0] + date[1] + date[2] + time[0] + time[1] + time[2] + '_' + this.state.computerName + '_Lablock.cfg' 
        // console.log(filename)
        return filename
    }

    handleClose() {
        this.setState({showErrorModal: false, showExportSuccessModal: false, showImportSuccessModal: false})
    }

    handleErrorShow() {
        this.handleClose()
        this.setState({showErrorModal: true})  
    }     

    handleExportSuccessShow() {
        this.handleClose()
        this.setState({showExportSuccessModal: true})  
    }     

    handleImportSuccessShow() {
        this.handleClose()
        this.setState({showImportSuccessModal: true})  
    }     

    handleFileSelected(e) {
        const files = Array.from(e.target.files)

        const reader = new FileReader()
        reader.onload = async (e) => { 
            let content = (e.target.result)
            // console.log(content)
            this.setState({Base64Content: content}) 
        };
        reader.readAsText(e.target.files[0])   
    }

    handleImport(data) {

        if (this.state.Base64Content) {
            data.Base64Content = this.state.Base64Content
            data.FileName = "lablockExport.cfg"

            let payload = {
                "ImportFile" : data
            }

            const importEndpoint = this.props.serviceURL + '/importlocal'
            console.log(importEndpoint)
            
            payload = AES.Encrypt(JSON.stringify(payload), this.props.JAKO, this.props.OJAK)
            fetch(importEndpoint, {method: 'POST', body: payload})
            .then(response => {
                if (response.status == '500') {
                    throw new Error("Error while importing file");
                }
                else {
                    return response.text()
                }
            })
            .then(response => {
                let responseDecoded = JSON.parse(AES.Decrypt(response, this.props.JAKO, this.props.OJAK))
                console.log(responseDecoded)            
                this.handleImportSuccessShow()
                // log outcome
                log(this.props.userObj.username, this.props.logModule, "imported the configuration file with the identification number MD5 "+ responseDecoded.ImportFileMD5 +" from ", this.state.host)

            })       
            .catch(error => {
                this.handleErrorShow()
                console.error('There has been a problem with your fetch operation:', error);
            });   
        }
    }


    render(){
        return (
            <>  

               <Jumbotron>
                    <Container>
                        <h4>Export 
                            <OverlayTrigger placement="right" overlay={<Tooltip id="export-tooltip">Export option will export the current restrictions and settings configurations to a file</Tooltip>}>
                                <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                    <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                </svg>
                            </OverlayTrigger>                               
                        </h4>
                        <p>When exporting you will dump the existing LabLock restrictions and settings configuration into a file so you can share it with other computers</p>
                        <Form.Group as={Row} controlId="export">
                            {/* <Form.Label column sm="3">Export configuration to path:</Form.Label>                            
                            <Col sm={{ span: 4, offset: 0 }}>                                
                                <Form.Control placeholder="C:/path" name="path" onChange={this.changePath}/>                                
                            </Col> */}
                            <Col sm={{ span: 0, offset: 0 }}>
                                <Button variant="dark" size="lg" onClick={this.export}>Export</Button>
                            </Col>                            
                        </Form.Group>
                    </Container>
                </Jumbotron>
                <Jumbotron>
                    <Container>
                        <h4>Import 
                            <OverlayTrigger placement="right" overlay={<Tooltip id="import-tooltip">Import option will import existing configurations from a file</Tooltip>}>
                                <svg width="0.8em" height="1.8em" viewBox="0 10 16 16" className="bi bi-info-circle-fill" fill="grey" xmlns="http://www.w3.org/2000/svg">
                                    <path fillRule="evenodd" d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412l-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z" />
                                </svg>
                            </OverlayTrigger>                               
                        </h4>
                        <Form.Group as={Row} controlId="importFile">
                            <Form.Label column sm="3">Import configuration from file</Form.Label>
                            <Col sm="5">
                                {/* <Form.File 
                                    name = "importFile"
                                    label=""
                                    accept="cfg" 
                                    data-browse="Choose file"
                                    custom 
                                    onChange={this.handleFileSelected} 
                                />      */}
                                <div className="custom-file">
                                    <input name="importFile" id="importFile" type="file" data-browse="Choose file" className="custom-file-input" onChange={this.handleFileSelected} />
                                    <label className="custom-file-label" htmlFor="inputGroupFile01">Choose file</label>
                                </div>
                            </Col>
                        </Form.Group> 

                        <br/>
                        
                        <Formik
                            onSubmit={(values) => {
                                console.log(values)
                                this.handleImport(values);
                            }}
                            initialValues={this.state.initialValues}
                        >
                        {({
                        handleSubmit,
                        handleChange,
                        handleBlur,
                        values,
                        touched,
                        isValid,
                        errors,
                        }) => (
                            <Form onSubmit={handleSubmit}>                          
                                <Form.Group as={Row} >
                                    <Col sm={{ span: 0, offset: 0 }}>
                                    <h6>Restrictions</h6>
                                    </Col>
                                    <Col sm={{ span: 0, offset: 0 }}>
                                    <h6>Settings</h6>
                                    </Col>
                                </Form.Group>                         
                                <Form.Group as={Row} >
                                    <Col sm={{ span: 0, offset: 1 }}>
                                        <Form.Check type="switch" id="ksettings" onChange={handleChange} label="Core" defaultChecked="true"/>
                                    </Col>
                                    <Col sm={{ span: 0, offset: 1 }}>
                                        <Form.Check type="switch" id="lablockData" value={values.lablockData} onChange={handleChange} label="Roles" defaultChecked="true"/>
                                    </Col>                            
                                </Form.Group> 
                                <Form.Group as={Row} >
                                    <Col sm={{ span: 0, offset: 1 }}>
                                        <Form.Check type="switch" id="ksyskeys" value={values.ksyskeys} onChange={handleChange} label="System keys" defaultChecked="true"/>
                                    </Col>
                                    <Col sm={{ span: 0, offset: 1 }}>
                                        <Form.Check type="switch" id="LDAPSettings" onChange={handleChange} label="AD settings" defaultChecked="true"/>
                                    </Col>                            
                                </Form.Group> 
                                <Form.Group as={Row} >
                                    <Col sm={{ span: 0, offset: 1 }}>
                                        <Form.Check type="switch" id="kappblocker" onChange={handleChange} label="Apps" defaultChecked="true"/>
                                    </Col>
                                    <Col sm={{ span: 0, offset: 1 }}>
                                        <Form.Check type="switch" id="LabLockSettings" onChange={handleChange} label="General" defaultChecked="true"/>
                                    </Col>                            
                                </Form.Group> 
                                <Form.Group as={Row} >
                                    <Col sm={{ span: 0, offset: 1 }}>
                                        <Form.Check type="switch" id="kstartup" onChange={handleChange} label="Startup" defaultChecked="true"/>
                                    </Col>
                                </Form.Group>                         
                                <Form.Group as={Row} >
                                    <Col sm={{ span: 0, offset: 1 }}>
                                        <Form.Check type="switch" id="kfirewall" onChange={handleChange} label="Firewall" defaultChecked="true"/>
                                    </Col>
                                    <Col sm={{ span: 0, offset: 0 }}>
                                        <Button variant="dark" size="lg" type="submit">Import now</Button>
                                    </Col>
                                </Form.Group>   
                            </Form>
                            )}
                        </Formik>

                        <Modal show={this.state.showExportSuccessModal} onHide={this.handleClose} aria-labelledby="contained-modal-title-vcenter" centered>
                            <Modal.Header closeButton>
                                <Modal.Title>Export</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>Restrictions and settings configuration exported successfully</Modal.Body>
                            <Modal.Footer>
                                <Button id="btn-close-export" variant="secondary" onClick={this.handleClose}>
                                Close
                                </Button>
                            </Modal.Footer>
                        </Modal>

                        <Modal show={this.state.showImportSuccessModal} onHide={this.handleClose} aria-labelledby="contained-modal-title-vcenter" centered>
                            <Modal.Header closeButton>
                                <Modal.Title>Import</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>Restrictions and settings configuration imported successfully</Modal.Body>
                            <Modal.Footer>
                                <Button id="btn-close-import" variant="secondary" onClick={this.handleClose}>
                                Close
                                </Button>
                            </Modal.Footer>
                        </Modal>

                        <Modal show={this.state.showErrorModal} onHide={this.handleClose} aria-labelledby="contained-modal-title-vcenter" centered>
                            <Modal.Header>
                                <Modal.Title>Ops, something didn't work as expected</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>There has been an error when processing your request, please try again or contact support</Modal.Body>
                            <Modal.Footer>
                                <Button variant="secondary" onClick={this.handleClose}>
                                Close
                                </Button>
                            </Modal.Footer>
                        </Modal>                         
                    </Container>
                </Jumbotron>
            </>
        );
  } 
}

export default ImportExportContainer;