import Button from 'react-bootstrap/Button'
import Container from 'react-bootstrap/Container'
import Jumbotron from 'react-bootstrap/Jumbotron'
import Form from 'react-bootstrap/Form'
import FormControl from 'react-bootstrap/FormControl'
import Row from 'react-bootstrap/Row'
import Col from 'react-bootstrap/Col'
import InputGroup from 'react-bootstrap/InputGroup'
import Modal from 'react-bootstrap/Modal'
import * as yup from 'yup';
import { Formik } from 'formik';
import log from '../../lib/log'
import AES from '../../lib/aes'


class Ldap extends React.Component {
	constructor(props) {
		super(props);

        let schema = yup.object({
            user: yup.string().required(),
            pass: yup.string().required(),
            baseDN: yup.string().required(),
            port: yup.number().min(0).max(65535).required(),
          });

        if (this.props.ldapSettings != null) {
            this.state = {
                showApplyModal: false,
                showErrorModal: false,
                showErrorADModal: false,
                schema: schema,
                button: 'none',
                ldapSettings: this.props.ldapSettings,
                user: this.props.ldapSettings.user,
                pass: this.props.ldapSettings.pass,
                baseDN: this.props.ldapSettings.baseDN,
                port: this.props.ldapSettings.port,
            };
        } else {
            this.state = {
                user: '',
                pass: '',
                baseDN: '',
                port: '',
                ldapSettings: {'user': '', 'pass':'', 'baseDN':'', 'port':''}
            };
        }
    console.log(this.state)
        // bind the action handlers
        this.handleApplyClose = this.handleApplyClose.bind(this);
        this.handleApplyShow = this.handleApplyShow.bind(this);  
        this.handleApplyADSettings = this.handleApplyADSettings.bind(this);
        this.handleErrorClose = this.handleErrorClose.bind(this);
        this.handleErrorShow = this.handleErrorShow.bind(this);    
        this.testConnection = this.testConnection.bind(this);      
    }

	componentDidMount(){
		var getUrl = window.location;		
		console.log(getUrl.host)
		var hostIP = getUrl.host
		this.setState(state => ({host: hostIP}))
    }

    handleApplyClose() {
        this.setState({showApplyModal: false})
    }

    handleApplyShow() {
        this.setState({showApplyModal: true})
    }

    handleErrorClose() {
        this.setState({showErrorModal: false, showErrorADModal: false})
    }

    handleErrorShow() {
        this.setState({showErrorModal: true})  
    }  

    handleErrorADShow() {
        this.setState({showErrorADModal: true})  
    } 

    // handleTextValue(e, value) {
    //     console.log(e.target.value)
    //     this.setState(state => ({value: e.target.value})) 
    // }

    handleUsername(e) {
        console.log(e.target.value)
        this.setState(state => ({user: e.target.value})) 
    }    

    handlePassword(e) {
        console.log(e.target.value)
        this.setState(state => ({pass: e.target.value})) 
    } 

    handleBasedn(e) {
        console.log(e.target.value)
        this.setState(state => ({baseDN: e.target.value})) 
    } 

    handlePort(e) {
        console.log(e.target.value)
        this.setState(state => ({port: e.target.value})) 
    }             


    handleApplyADSettings(data) {
        
        // transform checkboxes value to integer
        console.log(data)

        // build URL from baseDN
        let url = this.getADURL(this.state.baseDN)

        let payload = {
            'user': this.state.user,
            'pass': this.state.pass,
            'baseDN': this.state.baseDN, 
            'port': this.state.port,
            'url': url,   
        };

        const logDetails = {
            'title': 'Active Directory User Details',
            'columns' : [
                {
                    'title': 'Username',
                    'value': this.state.user
                },
                { 
                    'title': 'BaseDN',
                    'value': this.state.baseDN
                },
                {
                    'title': 'Port',
                    'value': this.state.port
                }
            ]
        };

        const ADLoginEndpoint = '/api/ldap/authenticate'
        fetch(ADLoginEndpoint, {method: 'POST', headers: {'Accept': 'application/json', 'Content-Type': 'application/json'}, body: JSON.stringify(payload)})
        .then(response => {
            console.log(response.status)
            switch(response.status){
                case 200:
                    const ADSettingsEndpoint = this.props.serviceURL + '/ldapsettings'
                    console.log(ADSettingsEndpoint)
              
                    payload = AES.Encrypt(JSON.stringify(payload), this.props.JAKO, this.props.OJAK)
                    fetch(ADSettingsEndpoint, {method: 'POST', body: payload})
                    .then(response => {
                        if (response.status == 200) {
                            this.handleApplyShow()
                            if (this.props.ldapSettings.user != '')
                                log(this.props.user.username, this.props.logModule, " modified Active Directory settings from ", this.state.host, logDetails)
                            else 
                                log(this.props.user.username, this.props.logModule, " added Active Directory settings from ", this.state.host, logDetails)    
                        } else {
                            this.handleErrorShow()
                        }
                    })       
                    .catch(error => {
                        this.handleErrorShow()
                        console.error('There has been a problem with your fetch operation:', error);
                    });              
                    break
                default:
                    this.handleErrorADShow()
                    break
            }
        })
    }

    getADURL(basedn) {
        let baseDNComponents = basedn.split(',')
        let baseDNBit = []
        let baseURL = []
        baseDNComponents.map((value) => {
            baseDNBit = value.split('='); 
            console.log(baseDNBit)
            if (baseDNBit[0].toLowerCase() == 'dc') {
                baseURL.push(baseDNBit[1])
                console.log(baseURL)
            }
        });
        let url = 'ldap://' + baseURL.join('.')          
        return url;
    }
    
    testConnection(){
        const ADLoginEndpoint = '/api/ldap/authenticate'
        console.log(ADLoginEndpoint)

        // build URL from baseDN
        let url = this.getADURL(this.state.baseDN)

        var payloadAuthentication = {
            'user': this.state.user,
            'pass': this.state.pass,
            'baseDN': this.state.baseDN, 
            'port': this.state.port,
            'url': url,             
        }

        fetch(ADLoginEndpoint, {method: 'POST', headers: {'Accept': 'application/json', 'Content-Type': 'application/json'}, body: JSON.stringify(payloadAuthentication)})
        .then(response => {
            console.log(response.status)
            switch(response.status){
                case 200:
                    this.setState(state => ({button: 'success'}))
                    break
                default:
                    this.setState(state => ({button: 'failed'}))
                    break
            }
        })
        .catch(err => console.log(err));        
    }  

    render(){
        return (
            <>     
                <Jumbotron>
                    <Container>
                        <Formik
                                validationSchema={this.state.schema}
                                onSubmit={(values) => {
                                    console.log(values)
                                    this.handleApplyADSettings(values);
                                }}
                                initialValues={this.state.ldapSettings}
                            >
                            {({
                            handleSubmit,
                            handleChange,
                            handleBlur,
                            values,
                            touched,
                            isValid,
                            errors,
                            }) => (
                            <Form onSubmit={handleSubmit}>                         
                                <Form.Group as={Row} controlId="user">
                                    <Form.Label column sm="4">Username</Form.Label>
                                    <Col sm="4">
                                        <Form.Control 
                                            name="user"
                                            value={values.user}
                                            onChange={(e) => {handleChange(e); this.handleUsername(e)}}
                                            onBlur={handleBlur}
                                            placeholder="SamAccountName"
                                            isInvalid={!!errors.user && touched.user}
                                        />
                                        <Form.Control.Feedback type="invalid">{errors.user}</Form.Control.Feedback>
                                    </Col>
                                </Form.Group>

                                <Form.Group as={Row} controlId="pass">
                                    <Form.Label column sm="4">Password</Form.Label>
                                    <Col sm="4">
                                        <Form.Control 
                                            type="password" 
                                            value={values.pass}
                                            onChange={(e) => {handleChange(e); this.handlePassword(e)}}
                                            onBlur={handleBlur}
                                            placeholder="AD Password"
                                            isInvalid={!!errors.pass && touched.pass}                                               
                                        />
                                        <Form.Control.Feedback type="invalid">{errors.pass}</Form.Control.Feedback>
                                    </Col>
                                </Form.Group>
                                <Form.Group as={Row} controlId="baseDN">
                                    <Form.Label column sm="4">BaseDN</Form.Label>
                                    <Col sm="4">
                                        <InputGroup className="mb-2">
                                            <FormControl 
                                                name="baseDN" 
                                                value={values.baseDN}
                                                onChange={(e) => {handleChange(e); this.handleBasedn(e)}}
                                                onBlur={handleBlur}
                                                placeholder="dc=domain,dc=com"
                                                isInvalid={!!errors.baseDN && touched.baseDN}
                                            />
                                            <Form.Control.Feedback type="invalid">{errors.baseDN}</Form.Control.Feedback>
                                        </InputGroup>                                
                                    </Col>
                                </Form.Group> 

                                <Form.Group as={Row} controlId="port">
                                    <Form.Label column sm="4">Port</Form.Label>
                                    <Col sm="4">
                                        <InputGroup className="mb-2">
                                            <FormControl 
                                                name="port" 
                                                value={values.port}
                                                onChange={(e) => {handleChange(e); this.handlePort(e)}}
                                                onBlur={handleBlur}
                                                placeholder="389 or 636"
                                                isInvalid={!!errors.port && touched.port}                                             
                                            />
                                            <Form.Control.Feedback type="invalid">{errors.port}</Form.Control.Feedback>
                                        </InputGroup>                                
                                    </Col>
                                </Form.Group> 

                                <Form.Group as={Row} controlId="port">
                                    <Form.Label column sm="4"></Form.Label>
                                    <Col sm="2">
                                        <Button id="btn-test-connection" onClick={this.testConnection} variant="info" >
                                            Test connection
                                        </Button>                              
                                    </Col>
                                    {this.state.button == 'success' ?
                                    <Col md={1}>	
                                        <Button variant="success">
                                            <svg width="1em" height="1em" viewBox="0 0 16 16" className="bi bi-check" fill="currentColor" xmlns="http://www.w3.org/2000/svg">
                                                <path fillRule="evenodd" d="M10.97 4.97a.75.75 0 0 1 1.071 1.05l-3.992 4.99a.75.75 0 0 1-1.08.02L4.324 8.384a.75.75 0 1 1 1.06-1.06l2.094 2.093 3.473-4.425a.236.236 0 0 1 .02-.022z"/>
                                            </svg>
                                        </Button>
                                    </Col>        
                                    : '' }
                                    {this.state.button == 'failed' ? 
                                    <Col md={1}>	
                                        <Button variant="danger">
                                            <svg width="1em" height="1em" viewBox="0 0 16 16" className="bi bi-x" fill="currentColor" xmlns="http://www.w3.org/2000/svg">
                                                <path fillRule="evenodd" d="M4.646 4.646a.5.5 0 0 1 .708 0L8 7.293l2.646-2.647a.5.5 0 0 1 .708.708L8.707 8l2.647 2.646a.5.5 0 0 1-.708.708L8 8.707l-2.646 2.647a.5.5 0 0 1-.708-.708L7.293 8 4.646 5.354a.5.5 0 0 1 0-.708z"/>
                                            </svg>
                                        </Button>
                                    </Col>        
                                    : '' }                                                 
                                </Form.Group>                                 

                                <Col md={{ span: 1, offset: 4 }}>
                                    <Button id="btn-apply-ldapSettings" type="submit" variant="dark" >
                                        Apply
                                    </Button>	
                                </Col>	
                            </Form>
                        )}
                        </Formik>                            

                        <Modal show={this.state.showApplyModal} onHide={this.handleApplyClose} aria-labelledby="contained-modal-title-vcenter" centered>
                            <Modal.Header closeButton>
                                <Modal.Title>AD settings</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>AD settings applied successfully</Modal.Body>
                            <Modal.Footer>
                                <Button id="btn-cancel-ldapsettings" variant="secondary" onClick={this.handleApplyClose}>
                                Close
                                </Button>
                            </Modal.Footer>
                        </Modal>

                        <Modal show={this.state.showErrorModal} onHide={this.handleErrorClose} aria-labelledby="contained-modal-title-vcenter" centered>
                            <Modal.Header>
                                <Modal.Title>Ops, something didn't work as expected</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>There has been an error when processing your request, please try again or contact support</Modal.Body>
                            <Modal.Footer>
                                <Button variant="secondary" onClick={this.handleErrorClose}>
                                Close
                                </Button>
                            </Modal.Footer>
                        </Modal>      

                        <Modal show={this.state.showErrorADModal} onHide={this.handleErrorClose} aria-labelledby="contained-modal-title-vcenter" centered>
                            <Modal.Header>
                                <Modal.Title>AD setup error</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>The settings you added can't be validated. Please try again.</Modal.Body>
                            <Modal.Footer>
                                <Button variant="secondary" onClick={this.handleErrorClose}>
                                Close
                                </Button>
                            </Modal.Footer>
                        </Modal>                                              
                    </Container>
                </Jumbotron>
            </>
        );
  } 
}

export default Ldap;