import Form from 'react-bootstrap/Form'
import Button from 'react-bootstrap/Button'
import Image from 'react-bootstrap/Image'
import Row from 'react-bootstrap/Row'
import Col from 'react-bootstrap/Col'
import Modal from 'react-bootstrap/Modal'
import * as yup from 'yup';
import { Formik } from 'formik';
import { withRouter } from 'next/router'
import AES from '../../lib/aes'
import log from '../../lib/log'
import LinearProgress from '@material-ui/core/LinearProgress';


class LoginForm extends React.Component {

    constructor(props) {
        super(props);
        console.log(props)
        let schema = yup.object({
            username: yup.string().required(),
            password: yup.string().required(),
        });

        this.state = {
          username: '',
          password: '',
          loginType: 'local',
          showProgress: false,
          showLoginErrorLockedModal: false,
          showLoginErrorAuthenticationModal: false,
          showLoginErrorPermissionsModal: false,
          showLoginErrorMultipleRolesModal: false,
          schema: schema,
          logModule: 'login'
        };
        
        this.handleLoginErrorClose = this.handleLoginErrorClose.bind(this);
        this.handleLoginErrorAuthenticationShow = this.handleLoginErrorAuthenticationShow.bind(this);
        this.handleLoginErrorLockedShow = this.handleLoginErrorLockedShow.bind(this);
        this.handleLoginErrorPermissionsShow = this.handleLoginErrorPermissionsShow.bind(this);
        this.handleLoginErrorMultipleRolesShow = this.handleLoginErrorMultipleRolesShow.bind(this);
        this.handleLogin = this.handleLogin.bind(this);

        // get groups and permissions depending on the user type
        // if (this.state.loginType == 'local'){
        fetch('/api/roles')
        .then(res => res.json())
        .then(data => {
            this.setState(state => ({groupList: data}))
            console.log(data)
        }) 
        .catch(err => console.log(err))    
    }

    componentDidMount() {
		var getUrl = window.location;		
		console.log(getUrl.host)
		var hostIP = getUrl.host
		this.setState(state => ({host: hostIP}))        
    }    
   
    handleLogin(data){

        this.setState(state => ({showProgress: true}))     
        let payload = {
            'user': data.username, 
            'pass': data.password, 
        };        
        var numberGroups = 0
        // console.log( this.props.JAKO)
        // add logic if possible to check if it is an AD or local user
        if (data.loginType == 'local'){
            // localuser call
            payload = AES.Encrypt(JSON.stringify(payload), this.props.JAKO, this.props.OJAK)
            const localLoginEndpoint =  this.props.serviceURL + '/localuservalidate'
            console.log(localLoginEndpoint)
    
            fetch(localLoginEndpoint, {method: 'POST', body: payload})
            .then(response => response.text())
            .then(response => {
                //  console.log(response)
                let responseDecoded = JSON.parse(AES.Decrypt(response, this.props.JAKO, this.props.OJAK))
                console.log(responseDecoded)
                if (responseDecoded.isValid == false) {
                    const localUsersEndpoint = this.props.serviceURL + '/localusers'
                    fetch(localUsersEndpoint)
                    .then(res => res.text())
                    .then(response => {
                        let responseDecoded = JSON.parse(AES.Decrypt(response, this.props.JAKO, this.props.OJAK))
                        responseDecoded.users.map((user) => {      
                            if (user.Name.trim().toLowerCase() == data.username.trim().toLowerCase()) {
                                if (user.Lockout == true) {
                                    this.handleLoginErrorLockedShow()
                                    // log outcome
                                    log(data.username, this.state.logModule, " is locked", this.state.host)
                                    return
                                } else {
                                    this.handleLoginErrorAuthenticationShow()
                                    // log outcome
                                    log(data.username, this.state.logModule, " User or password is incorrect", this.state.host)
                                    return
                                }
                            }
                        })
                    })
                    .catch(error => {
                        console.error('There has been a problem with your fetch operation:', error);
                    }); 
                    
                } else {
                    // get local groups for the user
                    let payloadUser = {'user': data.username}
                    payloadUser = AES.Encrypt(JSON.stringify(payloadUser), this.props.JAKO, this.props.OJAK)
                    const userMemberOfEndpoint = this.props.serviceURL + '/usermemberof'
                    fetch(userMemberOfEndpoint, {method: 'POST', body: payloadUser})
                    .then(response => response.text())
                    .then(response => {
                        var group                         
                        // compare with local groups permissions
                        console.log(response)
                        responseDecoded = JSON.parse(AES.Decrypt(response, this.props.JAKO, this.props.OJAK))
                        console.log(responseDecoded)                        
                        responseDecoded.Groups.map((userGroup) => {
                            this.state.groupList.map((localGroup) => {
                                if (userGroup.Name == localGroup.group_name && localGroup.group_type == 'local' && localGroup.status == 1) {
                                    group = localGroup
                                    numberGroups = numberGroups+1
                                    return
                                }
                            })
                        })
                        // check if there are more groups with permissions 
                        let permissions = []
                        if (numberGroups == 1) {
                            // populate permissions
                            group.dashboard == 1 ? permissions.push("dashboard") : ''
                            group.restrictions == 1 ? permissions.push("restrictions") : ''
                            group.activity == 1 ? permissions.push("activity") : ''
                            group.settings == 1 ? permissions.push("settings") : ''
                            group.import_export == 1 ? permissions.push("import_export") : ''

                            console.log(permissions)
                        } else if (numberGroups == 0) {
                            this.handleLoginErrorPermissionsShow()
                            log(data.username, this.state.logModule, " doesn’t have permissions to login to lablock from  ", this.state.host)                            
                            return
                        } else {
                            this.handleLoginErrorMultipleRolesShow()
                            log(data.username, this.state.logModule, "  is part of two or more groups", this.state.host)                            
                            return
                        }

                        if (permissions.length == 0) {
                            this.handleLoginErrorPermissionsShow()
                            log(data.username, this.state.logModule, " doesn’t have permissions to login to lablock from  ", this.state.host)                            
                            return
                        }

                        // retrieve more user information
                        let userDetails = {}
                        const localUsersEndpoint = this.props.serviceURL + '/localusers'
                        fetch(localUsersEndpoint)
                        .then(res => res.text())
                        .then(response => {
                            responseDecoded = JSON.parse(AES.Decrypt(response, this.props.JAKO, this.props.OJAK))
                            responseDecoded.users.map((user) => {      
                                //  console.log(user.Name) 
                                // console.log(data.username)
                                if (user.Name.trim().toLowerCase() == data.username.trim().toLowerCase()) {
                                    if (user.Lockout == true) {
                                        this.handleLoginErrorLockedShow()
                                        return
                                    }
                                    else {
                                        userDetails = {
                                            username: user.Name,
                                            fullname: user.FullName,
                                            role: group.name
                                        }
                                    }
                                }                                
                            })                           
                            console.log(userDetails)

                            // save session & permissions
                            let payloadSession = {
                                'user': userDetails,
                                'permissions':permissions,
                            }
                            fetch('/api/sessions', {method: 'POST', headers: {'Accept': 'application/json', 'Content-Type': 'application/json'}, body: JSON.stringify(payloadSession)})
                            .then(response => {
                                // log outcome
                                log(data.username, this.state.logModule, " logged in from ", this.state.host)

                                // route to the first permission set
                                let page = permissions.shift()
                                this.props.router.push(page);                                 
                            })
                            .catch(err => console.log(err));                              
                        })                      
                    })
                    .catch(err => console.log(err));
                }
          
            })       
            .catch(error => {
                console.error('There has been a problem with your fetch operation:', error);
            }); 
        } else if (data.loginType == 'AD'){
            const ADUserExistsEndpoint = '/api/ldap/user/exists'
            console.log(ADUserExistsEndpoint)

            var payloadAuthentication = {
                'user': data.username,
                'pass': data.password,
            }
            fetch(ADUserExistsEndpoint, {method: 'POST', headers: {'Accept': 'application/json', 'Content-Type': 'application/json'}, body: JSON.stringify(payloadAuthentication)})
            .then(response => {
                console.log(response)
                if (response.status != '200') {
                    this.handleLoginErrorAuthenticationShow()
                    // log outcome
                    log(data.username, this.state.logModule, " User or password is incorrect", this.state.host)
                    return
                } else {
                    fetch('/api/ldap/user/' + data.username)
                    .then(response => {
                        if (response.status != '200') {
                            this.handleLoginErrorLockedShow()
                            return
                        } else {
                            const ADLoginEndpoint = '/api/ldap/authenticate'
                            console.log(ADLoginEndpoint)
                            fetch(ADLoginEndpoint, {method: 'POST', headers: {'Accept': 'application/json', 'Content-Type': 'application/json'}, body: JSON.stringify(payloadAuthentication)})
                            .then(response => {
                                console.log(response)
                                if (response.status != '200') {
                                    this.handleLoginErrorAuthenticationShow()
                                    // log outcome
                                    log(data.username, this.state.logModule, " User or password is incorrect", this.state.host)
                                    return
                                } else {

                                    // get local groups for the user
                                    const userMemberOfEndpoint = '/api/ldap/groups/' + data.username
                                    fetch(userMemberOfEndpoint)
                                    .then(response => response.json())
                                    .then(response => {
                                        var group 
                                        // compare with local groups permissions
                                        console.log(response)
                                        response.map((userGroup) => {
                                            console.log(userGroup.dn)
                                            this.state.groupList.map((localGroup) => {
                                                console.log(localGroup.group_name)
                                                if (userGroup.dn.trim().toLowerCase() == localGroup.group_name.trim().toLowerCase() && localGroup.group_type == 'AD' && localGroup.status == 1) {
                                                    group = localGroup
                                                    numberGroups = numberGroups+1
                                                    return                                    
                                                } else {
                                                    // console.log(userGroup.dn)
                                                    let localGroupName = this.parseGroupCN(userGroup.dn)
                                                    console.log(localGroupName)
                                                    // if (userGroup.cn == localGroup.group_name  && localGroup.group_type == 'AD') {
                                                    if (localGroupName == localGroup.group_name && localGroup.group_type == 'AD' && localGroup.status == 1) {    
                                                        group = localGroup
                                                        numberGroups = numberGroups+1
                                                        return
                                                    }
                                                }
                                            })
                                        })
                                        // check if there are more groups with permissions 
                                        let permissions = []
                                        if (numberGroups == 1) {
                                            // populate permissions
                                            group.dashboard == 1 ? permissions.push("dashboard") : ''
                                            group.restrictions == 1 ? permissions.push("restrictions") : ''
                                            group.activity == 1 ? permissions.push("activity") : ''
                                            group.settings == 1 ? permissions.push("settings") : ''
                                            group.import_export == 1 ? permissions.push("import_export") : ''

                                            console.log(permissions)
                                        } else if (numberGroups == 0) {
                                            this.handleLoginErrorPermissionsShow()
                                            log(data.username, this.state.logModule, " doesn’t have permissions to login to lablock from  ", this.state.host)                            
                                            return
                                        } else {
                                            this.handleLoginErrorMultipleRolesShow()
                                            log(data.username, this.state.logModule, "  is part of two or more groups", this.state.host)                            
                                            return
                                        }

                                        if (permissions.length == 0) {
                                            this.handleLoginErrorPermissionsShow()
                                            log(data.username, this.state.logModule, " doesn’t have permissions to login to lablock from  ", this.state.host)                            
                                            return
                                        }

                                        let userDetails = {}
                                        fetch('/api/ldap/user/' + data.username)
                                        .then(res => res.json())
                                        .then(response => {
                                            userDetails = {
                                                username: response.sAMAccountName,
                                                fullname: response.displayName,
                                                role: group.name
                                            }

                                            // save session & permissions
                                            let payloadSession = {
                                                // 'user': data.username,
                                                'user': userDetails,
                                                'permissions':permissions,
                                            }
                                            fetch('/api/sessions', {method: 'POST', headers: {'Accept': 'application/json', 'Content-Type': 'application/json'}, body: JSON.stringify(payloadSession)})
                                            .then(response => {
                                                // log outcome
                                                log(userDetails.username, this.state.logModule, " logged in from ", this.state.host)

                                                // route to the first permission set
                                                let page = permissions.shift()
                                                this.props.router.push(page);   
                                            })
                                            .catch(err => console.log(err));                            
                                        }) 
                                        .catch(err => console.log(err));                      
                                    })
                                    .catch(err => console.log(err));
                                }
                            })
                            .catch(err => console.log(err));
                        }
                    }) 
                    .catch(err => console.log(err));                      
                }

            })       
            .catch(error => {
                console.error('There has been a problem with your fetch operation:', error);
            }); 
        }

    }

    parseGroupCN(basedn) {
        let baseDNComponents = basedn.split(',')
        let baseDNBit = []
        let groupCN = ""
        baseDNComponents.map((value) => {
            baseDNBit = value.split('=') 
            if (baseDNBit[0] == 'CN') {
                groupCN = baseDNBit[1]
                return
            }
        });      
        return groupCN;
    }

    handleLoginErrorAuthenticationShow(){
        this.setState({showLoginErrorAuthenticationModal: true, showProgress: false})
    }

    handleLoginErrorLockedShow(){
        this.setState({showLoginErrorLockedModal: true, showProgress: false})
    }

    handleLoginErrorPermissionsShow(){
        this.setState({showLoginErrorPermissionsModal: true, showProgress: false})
    }

    handleLoginErrorMultipleRolesShow(){
        this.setState({showLoginErrorMultipleRolesModal: true, showProgress: false})
    }

    handleLoginErrorClose(){
        this.setState({
            showLoginErrorLockedModal: false,
            showLoginErrorAuthenticationModal: false,
            showLoginErrorPermissionsModal: false,
            showLoginErrorMultipleRolesModal: false,
            showProgress: false
        })
    }
    render() {      
        return (
            <>              
            <div className="login-jumbotron">
                <Image src="lablock.jpeg" width="200px" className="image-logo" roundedCircle></Image>     
                {/* <h1>Lablock</h1> */}
                    <Formik
                            validationSchema={this.state.schema}
                            onSubmit={(values) => {
                                console.log(values)
                                this.handleLogin(values);                                
                            }}
                            initialValues={this.state}
                        >
                        {({
                        handleSubmit,
                        handleChange,
                        handleBlur,
                        values,
                        touched,
                        isValid,
                        errors,
                        }) => (
                        <Form onSubmit={handleSubmit}>          


                            <Form.Group as={Row} controlId="username">
                                {/* <Form.Label column sm="4">Username</Form.Label> */}
                                <Col >
                                    <Form.Control 
                                        // readOnly
                                        name="username"
                                        placeholder="Username"
                                        value={values.username}
                                        onChange={handleChange}  
                                    />
                                </Col>
                            </Form.Group>

                            <Form.Group as={Row} controlId="password">
                                {/* <Form.Label column sm="4">Password</Form.Label> */}
                                <Col >
                                    <Form.Control 
                                        type="password" 
                                        value={values.password}
                                        onChange={handleChange}  
                                        onBlur={handleBlur}
                                        placeholder="Password"
                                        isInvalid={!!errors.password && touched.password}                                               
                                    />
                                    <Form.Control.Feedback type="invalid">{errors.password}</Form.Control.Feedback>
                                </Col>
                            </Form.Group>
 
                            <Form.Group as={Row} controlId="loginType">
                                {/* <Form.Label column sm="4">Log on</Form.Label> */}
                                <Col >
                                    <Form.Control as="select" onChange={handleChange} custom >
                                        <option value="local">Local Authentication</option>
                                        <option value="AD">Active Directory</option>
                                    </Form.Control>
                                    <Form.Control.Feedback type="invalid">{errors.loginType}</Form.Control.Feedback>
                                </Col>
                            </Form.Group>
                            
                            <Button id="btn-login" type="submit" size="lg" block >
                                Log in
                            </Button>	
                                                     
                        </Form>
                        )}
                    </Formik>

                    {this.state.showProgress ?
                        // <CircularProgress disableShrink />
                        <div style={{
                            marginBottom: '3%'
                        }}>  
                        <br></br>
                        <LinearProgress />
                        </div>
                    : ''
                    }
                    <Modal show={this.state.showLoginErrorAuthenticationModal} onHide={this.handleLoginErrorClose} aria-labelledby="contained-modal-title-vcenter" centered>
                        <Modal.Header closeButton>
                            <Modal.Title>Login Error</Modal.Title>
                        </Modal.Header>
                        <Modal.Body>Incorrect user or password. Please try again.</Modal.Body>
                        <Modal.Footer>
                            <Button id="btn-cancel-errorLogin" variant="secondary" onClick={this.handleLoginErrorClose}>
                            Close
                            </Button>
                        </Modal.Footer>
                    </Modal>                

                    <Modal show={this.state.showLoginErrorMultipleRolesModal} onHide={this.handleLoginErrorClose} aria-labelledby="contained-modal-title-vcenter" centered>
                        <Modal.Header closeButton>
                            <Modal.Title>Login Error</Modal.Title>
                        </Modal.Header>
                        <Modal.Body>You belong to more than one role setup in Lablock. Please contact your administrator to fix this issue.</Modal.Body>
                        <Modal.Footer>
                            <Button id="btn-cancel-errorLogin" variant="secondary" onClick={this.handleLoginErrorClose}>
                            Close
                            </Button>
                        </Modal.Footer>
                    </Modal>    

                    <Modal show={this.state.showLoginErrorPermissionsModal} onHide={this.handleLoginErrorClose} aria-labelledby="contained-modal-title-vcenter" centered>
                        <Modal.Header closeButton>
                            <Modal.Title>Login Error</Modal.Title>
                        </Modal.Header>
                        <Modal.Body>You have no permissions to accesss Lablock.</Modal.Body>
                        <Modal.Footer>
                            <Button id="btn-cancel-errorLogin" variant="secondary" onClick={this.handleLoginErrorClose}>
                            Close
                            </Button>
                        </Modal.Footer>
                    </Modal>      

                    <Modal show={this.state.showLoginErrorLockedModal} onHide={this.handleLoginErrorClose} aria-labelledby="contained-modal-title-vcenter" centered>
                        <Modal.Header closeButton>
                            <Modal.Title>Login Error</Modal.Title>
                        </Modal.Header>
                        <Modal.Body>Your user is locked. Please try again later or contact your administrator.</Modal.Body>
                        <Modal.Footer>
                            <Button id="btn-cancel-errorLogin" variant="secondary" onClick={this.handleLoginErrorClose}>
                            Close
                            </Button>
                        </Modal.Footer>
                    </Modal>                                        
            </div>
            </>
        );
  } 
}

export default withRouter(LoginForm);