import Button from 'react-bootstrap/Button'
import Modal from 'react-bootstrap/Modal'
import OverlayTrigger from 'react-bootstrap/OverlayTrigger'
import Tooltip from 'react-bootstrap/Tooltip'
import log from '../../lib/log'

class ModalRestart extends React.Component {
  constructor(props) {
    super(props);
    // set state variables
    this.state = {
      showModal: false,
      user: this.props.user,
      logModule: this.props.logModule, 
    };
    // bind the action handlers
    this.reboot = this.reboot.bind(this);
    this.handleClose = this.handleClose.bind(this);
    this.handleShow = this.handleShow.bind(this);
    this.handleErrorClose = this.handleErrorClose.bind(this);
    this.handleErrorShow = this.handleErrorShow.bind(this);
  }

	componentDidMount(){
		var getUrl = window.location;		
		console.log(getUrl.host)
		var hostIP = getUrl.host
		this.setState(state => ({host: hostIP}))
  }

  handleClose() {
    this.setState({showModal: false})
  }

  handleShow() {
    this.setState({showModal: true})  
  }

  reboot() {
    // Reboot
      const rebootEndpoint = this.props.serviceURL + '/reboot'
      //console.log(rebootEndpoint)
      fetch(rebootEndpoint)      
      // .then(response => response.json())
      .then(response => {
        this.setState({showModal: false})
        // log outcome
        log(this.props.user.username, this.props.logModule, " restarted PC from ", this.state.host)
      })      
      .catch(error => {
        this.setState({showModal: false, showErrorModal: true})
        console.error('There has been a problem with your fetch operation:', error);
      }); 

  }

  handleErrorClose() {
    this.setState({showErrorModal: false})
  }

  handleErrorShow() {
      this.setState({showErrorModal: true})  
  }
  
  render() {
  
    return (
        <>  
          <OverlayTrigger placement="bottom" overlay={<Tooltip id="button-tooltip-2">Restart</Tooltip>}>     
            <Button id="btn-restart" variant="outline-info" onClick={this.handleShow}>
                <svg width="14em" height="2em" viewBox="0 0 16 16" className="bi bi-arrow-repeat" fill="currentColor" xmlns="http://www.w3.org/2000/svg">
                <path fillRule="evenodd" d="M2.854 7.146a.5.5 0 0 0-.708 0l-2 2a.5.5 0 1 0 .708.708L2.5 8.207l1.646 1.647a.5.5 0 0 0 .708-.708l-2-2zm13-1a.5.5 0 0 0-.708 0L13.5 7.793l-1.646-1.647a.5.5 0 0 0-.708.708l2 2a.5.5 0 0 0 .708 0l2-2a.5.5 0 0 0 0-.708z"/>
                <path fillRule="evenodd" d="M8 3a4.995 4.995 0 0 0-4.192 2.273.5.5 0 0 1-.837-.546A6 6 0 0 1 14 8a.5.5 0 0 1-1.001 0 5 5 0 0 0-5-5zM2.5 7.5A.5.5 0 0 1 3 8a5 5 0 0 0 9.192 2.727.5.5 0 1 1 .837.546A6 6 0 0 1 2 8a.5.5 0 0 1 .501-.5z"/>
                </svg>
            </Button>
          </OverlayTrigger>          
 
        <Modal show={this.state.showModal} onHide={this.handleClose} aria-labelledby="contained-modal-title-vcenter" centered>
          <Modal.Header closeButton>
            <Modal.Title>Restart Computer</Modal.Title>
          </Modal.Header>
          <Modal.Body>Are you sure you want to restart your computer?</Modal.Body>
          <Modal.Footer>
            <Button id="btn-cancel-restart" variant="secondary" onClick={this.handleClose}>
              Cancel
            </Button>
            <Button id="btn-apply-restart" variant="primary" onClick={this.reboot}>
              Continue
            </Button>
          </Modal.Footer>
        </Modal>

        <Modal show={this.state.showErrorModal} onHide={this.handleErrorClose} aria-labelledby="contained-modal-title-vcenter" centered>
            <Modal.Header closeButton>
                <Modal.Title>Ops, something didn't work as expected</Modal.Title>
            </Modal.Header>
            <Modal.Body>There has been an error when processing your request, please try again or contact support</Modal.Body>
            <Modal.Footer>
                <Button variant="secondary" onClick={this.handleErrorClose}>
                Close
                </Button>
            </Modal.Footer>
        </Modal>         
        </>
        );
  } 
}

export default ModalRestart;