import dynamic from 'next/dynamic'
import Desktop from './Desktop';
const SystemKeys = dynamic(() => import('./SystemKeys'))
import Apps from './Apps';
const Startup = dynamic(() => import('./Startup'))
import Firewall from './Firewall';
import RestrictionsControlMenu from './restrictionsControlMenu'
import Tabs from 'react-bootstrap/Tabs'
import Tab from 'react-bootstrap/Tab'
import Container from 'react-bootstrap/Container'

class RestrictionsContainer extends React.Component {
    constructor(props) {
        super(props);

        // split the rules between windows and custom ones
        let lablockRules = []
        let windowsRules = []
        if (this.props.firewallRules != null) {
          this.props.firewallRules.WinFirewallRules.map((rule) => {
              if (rule.isLabLock == true) {
                  lablockRules.push(rule) 
              } else if (!isNaN(rule.LocalPorts) && rule.LocalPorts != null) {
                  windowsRules.push(rule)
              }
          })         
          // console.log(this.props.syskeys)
          this.state = {
              restrictions: this.props.restrictions,
              syskeys: this.props.syskeys,
              startupPrograms: this.props.startupPrograms,
              applicationsAllowed: this.props.applications,
              firewallRules: lablockRules,
              windowsRules: windowsRules
          }
        } else {
          this.state = {
            restrictions: [],
            syskeys: [],
            startup: [],
            applicationsAllowed: [],
            firewallRules: [],
            windowsRules: [] 
          }
        }

        this.handleRestrictionsChange = this.handleRestrictionsChange.bind(this);
        this.handleSyskeysChange = this.handleSyskeysChange.bind(this);
        this.handleStartupProgramsChange = this.handleStartupProgramsChange.bind(this);
        this.handleApplicationsAllowedChange = this.handleApplicationsAllowedChange.bind(this);
        this.handleFirewallChange = this.handleFirewallChange.bind(this);
    }

    handleRestrictionsChange(e){
        let value

        console.log(this.state.restrictions.settings)
        console.log(e.target.id)
        // special restrictions textboxes 
        if (e.target.id == "SYS_AddFileExplorerPath" || e.target.id == "SYS_LockPC") { //|| e.target.id == "SYS_AddFileExplorerPathCheck") {            
          let k
          if (e.target.id) {
            k = this.state.restrictions.settings.findIndex(item => item.tag === e.target.id)
          }
          // remove existing restriction
          if (k >= 0) {
              this.state.restrictions.settings.splice([k], 1);
          }   
          // add new value
          let specialValue = e.target.value
          if (e.target.id == "SYS_LockPC") {
            // convert minutes to seconds
            specialValue *= 60
          }
          this.state.restrictions.settings.push({"tag":e.target.id,"value":specialValue});                                  
          console.log(specialValue)
        }        
        // e.target.checked contains true or false depending on the component select for both checkbox and tooglebuttons
        else if (e.target.checked === true) {
            this.props.components.configs.map((restriction) => {
                // in case of checkbox we get e.target.id
                if (restriction.restrictionTag == e.target.id) {
                  // when multiple registries need to be set, we pass the value 1
                  if (restriction.regSettings.length > 1) {
                    value = 1
                    this.state.restrictions.settings.push({"tag":e.target.id,"value":value});  
                  } else {
                    value = restriction.regSettings[0].value  
                    this.state.restrictions.settings.push({"tag":e.target.id,"value":value});                 
                  }
                  // in case of a tooglebutton we need to get e.target.value} 
                } else if (restriction.restrictionTag == e.target.value) {
                  value = restriction.regSettings[0].value  
                  this.state.restrictions.settings.push({"tag":e.target.value,"value":value});
                }
            });          
            console.log(value)
            
        } else if (e.target.checked === false) {
          // find checkbox on restrictions
          let j
          if (e.target.id == "SYS_AddFileExplorerPathCheck") {
            j = this.state.restrictions.settings.findIndex(item => item.tag === "SYS_AddFileExplorerPath")
          } else if (e.target.id) {
            j = this.state.restrictions.settings.findIndex(item => item.tag === e.target.id)
          // find tooglegroup on restrictions  
          } else if (e.target.value) {
            j = this.state.restrictions.settings.findIndex(item => item.tag === e.target.value)
          } 
          // remove restriction
          if (j >= 0) {
              this.state.restrictions.settings.splice([j], 1);
          }           
         } 
    }

    handleSyskeysChange(e){
        console.log(this.state.syskeys)
        console.log(e.target.value)
        if (e.target.checked == true) {
            this.state.syskeys.SystemKeys.push({"id":e.target.value});
        } else {
            let j = this.state.syskeys.SystemKeys.findIndex(item => item.id === e.target.value)
            if (j >= 0) {
                this.state.syskeys.SystemKeys.splice([j], 1);
            }
        }   
    }

    handleStartupProgramsChange(startupPrograms) {
      // this.console(this.state.startupPrograms)
        this.state.startupPrograms = startupPrograms
    }

    handleApplicationsAllowedChange(applicationsAllowed) {
      // this.console(this.state.applicationsAllowed)
        this.state.applicationsAllowed = applicationsAllowed
    }

    handleFirewallChange(firewallRules) {
        // console.log(this.state.firewallRules)
        this.state.firewallRules = firewallRules
        // console.log(this.state.firewallRules)
    }    
    render () {
  
      return (
        <>

          <Container>
                <Tabs defaultActiveKey="desktop" id="uncontrolled-tab-example">
                    <Tab eventKey="desktop" title="Core">
                      <Desktop restrictions={this.state.restrictions} components={this.props.components} handleRestrictionsChange={this.handleRestrictionsChange}/>
                    </Tab>
                    <Tab eventKey="systemkeys" title="System Keys">
                      <SystemKeys syskeys={this.state.syskeys} syskeysElements={this.props.syskeysElements} components={this.props.components} handleSyskeysChange={this.handleSyskeysChange}/>		    
                    </Tab>
                    <Tab eventKey="apps" title="Apps">
                      <Apps applicationsAllowed={this.state.applicationsAllowed} handleApplicationsAllowedChange={this.handleApplicationsAllowedChange} serviceURL={this.props.serviceURL} JAKO={this.props.JAKO} OJAK={this.props.OJAK}/>		    
                    </Tab>	  		  		  
                    <Tab eventKey="startup" title="Startup">
                      <Startup startupPrograms={this.state.startupPrograms} handleStartupProgramsChange={this.handleStartupProgramsChange} />
                    </Tab>		  
                    <Tab eventKey="firewall" title="Firewall">
                      <Firewall firewallRules={this.state.firewallRules} windowsRules={this.state.windowsRules} handleFirewallChange={this.handleFirewallChange} />
                    </Tab>                  
                </Tabs>

                <RestrictionsControlMenu restrictions={this.state.restrictions} components={this.props.components} syskeys={this.state.syskeys} applicationsAllowed={this.state.applicationsAllowed} startupPrograms={this.state.startupPrograms} firewallRules={this.state.firewallRules} serviceURL={this.props.serviceURL} user={this.props.userObj} JAKO={this.props.JAKO} OJAK={this.props.OJAK}/>
          </Container>
        </>  
      )
    }
}

export default RestrictionsContainer