import Form from 'react-bootstrap/Form'
import Button from 'react-bootstrap/Button'
import Container from 'react-bootstrap/Container'
import Jumbotron from 'react-bootstrap/Jumbotron'
import Table from 'react-bootstrap/Table'
import OverlayTrigger from 'react-bootstrap/OverlayTrigger'
import Tooltip from 'react-bootstrap/Tooltip'
import Modal from 'react-bootstrap/Modal'
import Row from 'react-bootstrap/Row'
import Col from 'react-bootstrap/Col'
import InputGroup from 'react-bootstrap/InputGroup'
import * as yup from 'yup';
import { Formik } from 'formik';
import { FaEdit } from 'react-icons/fa';
import log from '../../lib/log'
import AES from '../../lib/aes'
// import { DataGrid, GridRowsProp, GridColDef } from '@material-ui/data-grid';
import EditIcon from '@material-ui/icons/Edit';

class Roles extends React.Component {
	constructor(props) {
        super(props);
        
        let schema = yup.object({
            name: yup.string().required(),
            description: yup.string(),
            group_name: yup.string().required(),
            group_type: yup.string(),
            status: yup.string()
          });
        
        let rows = [
            { id: 1, name:'Admin', description: 'Admin role', group_name: 'Administrator', permissions:'Dashboard', status: '1'}
        ];          

        let columns = [
            { field: 'name', headerName: 'Role name', width: '150px' },
            { field: 'description', headerName: 'Description', width: '250px' },
            { field: 'group_name', headerName: 'Group', width: '150px' },
            { field: 'permissions', headerName: 'Permissions', width: '150px' },
            { field: 'status', headerName: 'Status', width: '100px' },
        ];          

        this.state = {
            roleList: [],
            role: {},
            columns: columns,
            rows: rows,
            schema: schema,
            showErrorSameNameGroup: false,
            disableSubmit: false,
            validationButton: 'outline-dark'
        }   

        // bind the action handlers
        this.handleAdd = this.handleAdd.bind(this);
        this.handleEdit = this.handleEdit.bind(this);
        this.handleSave = this.handleSave.bind(this);
        this.handleClose = this.handleClose.bind(this);
        this.handleShow = this.handleShow.bind(this);
        this.handleErrorClose = this.handleErrorClose.bind(this);
        this.handleErrorShow = this.handleErrorShow.bind(this);   
        this.testGroup = this.testGroup.bind(this)
        this.handleCheckboxChange = this.handleCheckboxChange.bind(this)
        // this.handleChangeValue = this.handleChangeValue.bind(this)
    }

    handleAdd(){
        // set title
        this.setState(state => ({
            role: {status: 1, dashboard: 0, restrictions: 0, settings: 0, activity: 0, import_export: 0, group_type: 'local'},
            validationButton: 'outline-dark', 
            disableSubmit: false, 
            status: 1
        }))
        this.handleShow()
    }

    handleEdit(e){

        // Get role
        let roleId = e.target.id
        if (!roleId) {
            roleId = e.target.parentNode.id;
        }        
        const roleEndpoint = '/api/role/' + roleId
        console.log(roleEndpoint)

        fetch(roleEndpoint)
        .then(res => res.json())
        .then(data => {            
            console.log(data)

            this.setState(state => ({
                role: data,
                groupName: data.group_name,
                groupType: data.group_type,
                status: data.status,
                dashboard: data.dashboard,
                restrictions: data.restrictions,
                activity: data.activity,
                settings: data.settings,
                import_export: data.import_export,
                disableSubmit: false
            }))
            console.log(data)
        }) 
        .catch(err => console.log(err));

        this.handleShow()       
    }

    handleClose() {
        this.setState({showModal: false, showErrorSameNameGroup: false, validationButton: 'outline-dark'})
    }
    
    handleShow() {           
        this.setState({showModal: true})  
    }    

    componentDidMount(){
        // fetch the roles from the DB
        fetch('/api/roles')
        .then(res => res.json())
        .then(data => {
            this.setState(state => ({roleList: data}))
            console.log(data)
        }) 
        .catch(err => console.log(err));

        // load localgroups when mounting  
        const localGroupsEndpoint = this.props.serviceURL + '/localgroups'
        console.log(localGroupsEndpoint)
        fetch(localGroupsEndpoint)
        .then(res => res.text())
        .then(data => {
            let decodedData = JSON.parse(AES.Decrypt(data, this.props.JAKO, this.props.OJAK))
            console.log(decodedData)
            this.setState(state => ({localgroups: decodedData})) 
        })
        .catch(err => console.log(err));
  
		var getUrl = window.location;		
		console.log(getUrl.host)
		var hostIP = getUrl.host
		this.setState(state => ({host: hostIP}))        
    }

    handleGroupType(e) {
        console.log(e.target.value)
        this.setState(state => ({groupType: e.target.value})) 
    }

    handleGroupName(e) {
        console.log(e.target.value)
        this.setState(state => ({groupName: e.target.value})) 
    }

    testGroup(){
        console.log(this.state)
        // initialize the button as an error
        this.setState(state => ({validationButton: 'danger'})) 

        if (this.state.groupType == 'local') {
            this.state.localgroups.groups.map((group => {
                if (group.Name == this.state.groupName) {
                    this.setState(state => ({validationButton: 'success'})) 
                }
            }))
        } 
        else if ((this.state.groupType == 'AD')) {
            const ldapGroupEndpoint = '/api/ldap/group/' + this.state.groupName
            console.log(ldapGroupEndpoint)
            fetch(ldapGroupEndpoint)
            .then(res => res.json())
            .then(response => {
                console.log(response)
                if (response.length > 0){
                    this.setState(state => ({validationButton: 'success'}))
                }
            })
            .catch(err => console.log(err));
        }
    }

    handleSave(data) {
        
        let error = false
        this.state.roleList.map(role => {
            if (role.name == data.name && role.id != this.state.role.id) {
                this.handleSameNameGroupErrorShow()
            }
            if (role.group_name == data.group_name && role.id != this.state.role.id) {
                this.handleSameNameGroupErrorShow()
            }            
        })

        if (this.state.showErrorSameNameGroup == false) {
            // transform checkboxes value to integer
            console.log(this.state)

            let payload = {
                'name': data.name, 
                'description': data.description ? data.description : '', 
                'group_name': data.group_name, 
                'group_type': data.group_type, 
                'status': this.state.status ? this.state.status : 0,
                'dashboard': this.state.dashboard ? this.state.dashboard : 0,
                'restrictions': this.state.restrictions ? this.state.restrictions : 0,
                'activity': this.state.activity ? this.state.activity : 0,
                'settings': this.state.settings ? this.state.settings : 0,
                'import_export': this.state.import_export ? this.state.import_export : 0,
            };

            let permissionsValue = this.state.dashboard ? "Dashboard " : '' 
            permissionsValue+= this.state.restrictions ? "Restrictions " : '' 
            permissionsValue+= this.state.activity ? "Activity " : '' 
            permissionsValue+= this.state.settings ? "Settings " : '' 
            permissionsValue+= this.state.import_export ? "ImportExport" : ''
            console.log(permissionsValue)
            let logDetails = {
                'title': 'Role Details',
                'columns': [
                    {
                        'title': 'Role Name',
                        'value': data.name
                    },
                    {
                        'title': 'Group Type',
                        'value': data.group_type
                    },
                    {
                        'title': 'Group Name',
                        'value': data.group_name,
                    },
                    {
                        'title': 'Permissions',
                        'value': permissionsValue
                    }
                ]
            }
            console.log(payload)

            if (!this.state.role.id) {
                const rolesEndpoint = '/api/roles'
                console.log(rolesEndpoint)

                if ((this.state.groupType == 'AD')) {
                    const ldapGroupEndpoint = '/api/ldap/group/' + this.state.groupName
                    console.log(ldapGroupEndpoint)
                    fetch(ldapGroupEndpoint)
                    .then(response => {
                        // if group exists, we save the results
                        if (response.status == 200) {

                            fetch(rolesEndpoint, {method: 'POST', headers: {'Accept': 'application/json', 'Content-Type': 'application/json'}, body: JSON.stringify(payload)})
                            .then(res => res.json())
                            .then(jsonResponse => {
                                console.log(jsonResponse)
                                log(this.props.user.username, this.props.logModule, " created the role "+ data.name +" based on " + data.group_type + " from ", this.state.host, logDetails)
                            }) 
                            .catch(err => console.log(err));
                        } else {
                            this.handleErrorADShow()
                        }
                    })
                } else {
                    fetch(rolesEndpoint, {method: 'POST', headers: {'Accept': 'application/json', 'Content-Type': 'application/json'}, body: JSON.stringify(payload)})
                    .then(res => res.json())
                    .then(jsonResponse => {
                        console.log(jsonResponse)
                        log(this.props.user.username, this.props.logModule, " created the role "+ data.name +" based on " + data.group_type + " from ", this.state.host, logDetails)
                        window.location.reload(false);
                    }) 
                    .catch(err => console.log(err));
                }
            } else if (this.state.role.id > 0) {
                let roleId = this.state.role.id
                const roleEndpoint = '/api/role/' + roleId
                console.log(roleEndpoint)

                if ((this.state.groupType == 'AD')) {
                    const ldapGroupEndpoint = '/api/ldap/group/' + this.state.groupName
                    console.log(ldapGroupEndpoint)
                    fetch(ldapGroupEndpoint)
                    .then(response => {
                        // if group exists, we save the results
                        if (response.status == 200) {
                            fetch(roleEndpoint, {method: 'PUT', headers: {'Accept': 'application/json', 'Content-Type': 'application/json'}, body: JSON.stringify(payload)})
                            .then(res => res.json())
                            .then(jsonResponse => {
                                console.log(jsonResponse)
                                // log outcome
                                log(this.props.user.username, this.props.logModule, " modified the role "+ data.name +" based on " + data.group_type + " from ", this.state.host, logDetails)
                                if (data.status != this.state.status) {
                                    if (data.status == true) {
                                        log(this.props.user.username, this.props.logModule, " disabled the role "+ data.name +" based on " + data.group_type + " from ", this.state.host)
                                    } else {
                                        log(this.props.user.username, this.props.logModule, " enabled the role "+ data.name +" based on " + data.group_type + " from ", this.state.host)
                                    }
                                }
                                // prevent submit
                                window.location.reload(false);

                            }) 
                            .catch(err => console.log(err));
                        } else {
                            this.handleErrorADShow()
                        }

                    })
                    .catch(err => console.log(err));
                } else {    
                    fetch(roleEndpoint, {method: 'PUT', headers: {'Accept': 'application/json', 'Content-Type': 'application/json'}, body: JSON.stringify(payload)})
                    .then(res => res.json())
                    .then(jsonResponse => {
                        console.log(jsonResponse)
                        
                        // log outcome
                        log(this.props.user.username, this.props.logModule, " modified the role "+ data.name +" based on " + data.group_type + " from ", this.state.host, logDetails)
                        if (data.status != this.state.status) {
                            if (data.status == true) {
                                log(this.props.user.username, this.props.logModule, " disabled the role "+ data.name +" based on " + data.group_type + " from ", this.state.host)
                            } else {
                                log(this.props.user.username, this.props.logModule, " enabled the role "+ data.name +" based on " + data.group_type + " from ", this.state.host)                                
                            }
                        }
                        // prevent submit
                        window.location.reload(false);                    
                    }) 
                    .catch(err => console.log(err));
                }
            }        
        }
    }

    checkboxToInteger(checkbox) {
        if (checkbox.length > 0 || checkbox == 1 || checkbox == true) {
            return 1;
        }
        return 0;
    }    
    
    handleErrorClose() {
        this.setState({showErrorModal: false, showErrorADModal: false, showErrorSameNameGroup: false})
    }
    
    handleErrorShow() {
        this.setState({showErrorModal: true, disableSubmit: false})  
    }

    handleSameNameGroupErrorShow() {
        this.setState({showErrorSameNameGroup: true, disableSubmit: false})  
    }    

    handleErrorADShow() {
        this.setState({showErrorADModal: true, disableSubmit: false})  
    } 

    disableSubmit() {
        this.setState({disableSubmit : true})
    }

    handleCheckboxChange(e) {
        // console.log(e.target.id)
        // console.log(this.checkboxToInteger(e.target.checked))
        this.setState({ [e.target.id]: this.checkboxToInteger(e.target.checked)})
    }

    render(){
        return (
            <>          
                <Jumbotron>
                    <Container>
                    
                        {!this.state.showModal ? 
                            <div style={{
                                marginBottom: '3%'
                            }}>                                  
                            <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip-2">Add new role</Tooltip>}> 
                                <Button id="btn-new-role" variant="outline-dark" onClick={this.handleAdd}>
                                    <svg width="1em" height="1em" viewBox="0 0 16 16" className="bi bi-person-plus" fill="currentColor" xmlns="http://www.w3.org/2000/svg">
                                        <path fillRule="evenodd" d="M8 5a2 2 0 1 1-4 0 2 2 0 0 1 4 0zM6 8a3 3 0 1 0 0-6 3 3 0 0 0 0 6zm6 5c0 1-1 1-1 1H1s-1 0-1-1 1-4 6-4 6 3 6 4zm-1-.004c-.001-.246-.154-.986-.832-1.664C9.516 10.68 8.289 10 6 10c-2.29 0-3.516.68-4.168 1.332-.678.678-.83 1.418-.832 1.664h10zM13.5 5a.5.5 0 0 1 .5.5V7h1.5a.5.5 0 0 1 0 1H14v1.5a.5.5 0 0 1-1 0V8h-1.5a.5.5 0 0 1 0-1H13V5.5a.5.5 0 0 1 .5-.5z"/>
                                    </svg>                            
                                </Button>
                            </OverlayTrigger>
                            </div>
                        : ''
                        }

                        {!this.state.showModal ?                        
                        // <div style={{ height: 250, width: '100%' }}>
                        //     <DataGrid rows={this.state.roleList} columns={this.state.columns} />
                        
                        <Table striped bordered hover>
                        <thead>
                            <tr>
                            <th>#</th>
                            <th>Role name</th>
                            <th>Description</th>
                            <th>Group</th>
                            <th>Permissions</th>
                            <th>Status</th>
                            <th></th>
                            </tr>
                        </thead>   
                        <tbody>
                            {this.state.roleList.map((role) =>
                            <tr key={role.id}>
                                <td>{role.id}</td>
                                <td>{role.name}</td>
                                <td>{role.description}</td>
                                <td>{role.group_name}</td>
                                <td>
                                    {role.dashboard ? "Dashboard" : ""}{role.dashboard ? <br></br>:''}
                                    {role.restrictions ? "Restrictions" : ""}{role.restrictions ? <br></br>:''}
                                    {role.settings ? "Settings" : ""}{role.settings ? <br></br>:''}
                                    {role.activity ? "Activity" : ""}{role.activity ? <br></br>:''}
                                    {role.import_export ? "Import/Export" : ""}</td>
                                <td>{role.status == 1 ? "active" : "deactivated"}</td>
                                <td>


                                    <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip-edit">Edit</Tooltip>}>
                                        <Button id={role.id} variant="variant-dark" onClick={this.handleEdit}>
                                            <FaEdit id={role.id} />
                                        </Button>
                                    </OverlayTrigger>
                                </td>
                            </tr>
                            )}
                        </tbody>         
                        </Table>
                        // </div>
                        // end listing
                        :
                        // start form
                        <Formik
                            enableReinitialize={true}
                            validationSchema={this.state.schema}
                            onSubmit={(values) => {
                              console.log(values)
                              this.disableSubmit();
                              this.handleSave(values);
                            }}
                            initialValues={this.state.role}
                        >
                        {({
                          handleSubmit,
                          handleChange,
                          handleBlur,
                          values,
                          touched,
                          isValid,
                          errors,
                        }) => (
                          <Form onSubmit={handleSubmit}>                        
                            <Form.Group as={Row} controlId="name">
                                <Form.Label column sm="2">Name</Form.Label>
                                <Col sm="10">
                                    <Form.Control 
                                        type="text"
                                        name="name"
                                        value={values.name}
                                        onChange={handleChange}
                                        onBlur={handleBlur}
                                        isInvalid={!!errors.name && touched.name}
                                    />
                                    <Form.Control.Feedback type="invalid">{errors.name}</Form.Control.Feedback>                                    
                                </Col>
                            </Form.Group>

                            <Form.Group as={Row} controlId="description">
                                <Form.Label column sm="2">Description</Form.Label>
                                <Col sm="10">
                                    <Form.Control 
                                        type="text"
                                        name="description"
                                        value={values.description}
                                        onChange={handleChange}
                                        isInvalid={!!errors.description}
                                    />
                                </Col>
                            </Form.Group> 

                            <Form.Group as={Row} controlId="group_type">
                                <Form.Label column sm="2">Windows group</Form.Label>
                                <Col sm="10">
                                    <Form.Control as="select" value={values.group_type} defaultValue={values.group_type} onChange={(e) => {handleChange(e); this.handleGroupType(e)}} custom >
                                        <option value="local">Local</option>
                                        <option value="AD">Active Directory</option>
                                    </Form.Control>
                                </Col>
                            </Form.Group>

                            <Form.Group as={Row} controlId="group_name">
                                <Form.Label column sm="2">Group name</Form.Label>
                                <Col sm="8">
                                    <Form.Control  
                                        type="text"
                                        name="group_name"
                                        value={values.group_name}
                                        onChange={(e) => {handleChange(e); this.handleGroupName(e)}}
                                        onBlur={handleBlur}
                                        isInvalid={!!errors.group_name && touched.group_name}                                    
                                    />
                                    <Form.Control.Feedback type="invalid">
                                        {errors.group_name}
                                    </Form.Control.Feedback>                                    
                                </Col>
                                <Col sm="2">
                                    <OverlayTrigger placement="right" overlay={<Tooltip id="button-tooltip-2">Validate group</Tooltip>}> 
                                        <Button id="btn-test-group" variant={this.state.validationButton} onClick={this.testGroup}>
                                            <svg width="1em" height="1em" viewBox="0 0 16 16" className="bi bi-check2" fill="currentColor" xmlns="http://www.w3.org/2000/svg">
                                                <path fillRule="evenodd" d="M13.854 3.646a.5.5 0 0 1 0 .708l-7 7a.5.5 0 0 1-.708 0l-3.5-3.5a.5.5 0 1 1 .708-.708L6.5 10.293l6.646-6.647a.5.5 0 0 1 .708 0z"/>
                                            </svg>                        
                                        </Button>
                                    </OverlayTrigger>
                                </Col>
                            </Form.Group> 

                            <Form.Group as={Row} controlId="status">
                                <Form.Label column sm="2">Active</Form.Label>
                                <Col sm="10">
                                        <input
                                            id="status"
                                            name="status"
                                            type="checkbox"
                                            defaultChecked={this.state.role.status}
                                            onClick={(e) => this.handleCheckboxChange(e)}
                                            // onChange={handleChange} 
                                            // onChange={(e) => {handleChange(e); this.handleCheckboxChange(e)}}
                                            />
                                </Col>
                            </Form.Group>       
 
                            <Table striped bordered hover>
                            <thead>
                                <tr>
                                    <th>Permissions</th>
                                    <th>Allow</th>
                                </tr>
                            </thead>   
                            <tbody>
                                <tr>
                                    <td>Dashboard</td>
                                    <td>								
                                        <input
                                            id="dashboard"
                                            name="dashboard"
                                            type="checkbox"
                                            defaultChecked={this.state.role.dashboard}
                                            onClick={(e) => this.handleCheckboxChange(e)}
                                            // onChange={handleChange} />                                          
                                            // onChange={(e) => {handleChange(e); this.handleCheckboxChange(e)}}
                                            />
                                    </td>
                                </tr>
                                <tr>
                                    <td>Restrictions</td>
                                    <td>								
                                        <input
                                            id="restrictions"
                                            name="restrictions"
                                            type="checkbox"
                                            defaultChecked={this.state.role.restrictions}
                                            onClick={(e) => this.handleCheckboxChange(e)}
                                            // onChange={handleChange} />                                        
                                            // onChange={(e) => {handleChange(e); this.handleCheckboxChange(e)}}
                                            />
                                    </td>
                                </tr>
                                <tr>
                                    <td>Activity</td>
                                    <td>								
                                        <input
                                            id="activity"
                                            name="activity"
                                            type="checkbox"
                                            defaultChecked={this.state.role.activity}
                                            value={values.activity}
                                            onClick={(e) => this.handleCheckboxChange(e)}
                                            // onChange={handleChange} />                                        
                                            // onChange={(e) => {handleChange(e); this.handleCheckboxChange(e)}}
                                            />
                                    </td>
                                </tr>
                                <tr>
                                    <td>Settings</td>
                                    <td>								
                                        <input
                                            id="settings"
                                            name="settings"
                                            type="checkbox"
                                            defaultChecked={this.state.role.settings}
                                            // onChange={handleChange} />        
                                            onClick={(e) => this.handleCheckboxChange(e)}
                                            // onChange={(e) => {handleChange(e); this.handleCheckboxChange(e)}}
                                            />
                                    </td>
                                </tr>
                                <tr>
                                    <td>Import/Export</td>
                                    <td>								
                                        <input 
                                            id="import_export"
                                            name="import_export"
                                            type="checkbox"
                                            defaultChecked={this.state.role.import_export}
                                            onClick={(e) => this.handleCheckboxChange(e)}
                                            // onChange={handleChange} />                                        
                                            // onChange={(e) => {handleChange(e); this.handleCheckboxChange(e)}}
                                            />
                                    </td>
                                </tr>                                
                            </tbody>
                            </Table>   
                            <Row>
                                <Col md={{ span: 1, offset: 4 }}>
                                    <Button id="btn-cancel" variant="secondary" onClick={this.handleClose}>
                                        Cancel
                                    </Button>
                                </Col>
                                <Col md={1}>
                                    <Button id="btn-save" disabled={this.state.disableSubmit} variant="primary" type="submit">
                                    Save
                                    </Button>    
                                </Col>        
                            </Row>                
                        </Form>
                    )}
                    </Formik>
                    }

                        <Modal show={this.state.showErrorSameNameGroup} onHide={this.handleErrorClose} aria-labelledby="contained-modal-title-vcenter" centered>
                            <Modal.Header closeButton>
                                <Modal.Title>Duplicate error</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>You have entered an existing role name or a group assigned to another role. Please double check and try again</Modal.Body>
                            <Modal.Footer>
                                <Button variant="secondary" onClick={this.handleErrorClose}>
                                Close
                                </Button>
                            </Modal.Footer>
                        </Modal>  


                        <Modal show={this.state.showErrorModal} onHide={this.handleErrorClose} aria-labelledby="contained-modal-title-vcenter" centered>
                            <Modal.Header closeButton>
                                <Modal.Title>Ops, something didn't work as expected</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>There has been an error when processing your request, please try again or contact support</Modal.Body>
                            <Modal.Footer>
                                <Button variant="secondary" onClick={this.handleErrorClose}>
                                Close
                                </Button>
                            </Modal.Footer>
                        </Modal>              

                        <Modal show={this.state.showErrorADModal} onHide={this.handleErrorClose} aria-labelledby="contained-modal-title-vcenter" centered>
                            <Modal.Header>
                                <Modal.Title>AD setup error</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>The settings you added can't be validated. Please try again.</Modal.Body>
                            <Modal.Footer>
                                <Button variant="secondary" onClick={this.handleErrorClose}>
                                Close
                                </Button>
                            </Modal.Footer>
                        </Modal>     

                    </Container>
                </Jumbotron>              
            </>
        );
  } 
}

export default Roles;