
import {Button, Alert, Table, Modal} from 'react-bootstrap'
import React, { useEffect, useState } from "react";
import AES from '../../lib/aes'
import log from '../../lib/log'

class SuspendRestrictions extends React.Component {
  constructor(props) {
    super(props);

    let suspendButtonDisabled = true;
    if (this.props.suspendProperties != null) {
      if (this.props.suspendProperties.suspend == false && this.props.restrictionsCount > 0) {
        suspendButtonDisabled = '';    
      }
    
      // set state variables
      this.state = {
        showModal: false,
        isSuspended: this.props.suspendProperties.suspend,
        suspendedOn: this.props.suspendProperties.suspendedOn,
        suspendButtonDisabled: suspendButtonDisabled,
        duration: this.props.suspendProperties.duration
      };
    }
    else {
      this.state = {
        isSuspended: '',
        suspendedOn: '',
        suspendedDate: '',
        duration: 0
      };
      

    }
  

    // bind the action handlers
    this.suspend = this.suspend.bind(this);
    this.resume = this.resume.bind(this);
    this.handleClose = this.handleClose.bind(this);
    this.handleShow = this.handleShow.bind(this);
  }

  componentDidMount(){
      this.timerID = setInterval(() => this.tick(), 1000);        
      var getUrl = window.location;		
      console.log(getUrl.host)
      var hostIP = getUrl.host
      this.setState(state => ({host: hostIP}))         
  }

  componentWillUnmount(){
      clearInterval(this.timerID);
  }

  tick() {
      console.log(this.state.isSuspended + ' ' + this.state.suspendButtonDisabled)
      if (this.props.restrictionsCount == 0) {
        this.setState(state => ({suspendedDate: 'No restrictions applied'}))
      }
      else if (this.props.restrictionsCount > 0 && this.state.isSuspended == false) {
        this.setState(state => ({suspendedDate: 'Restrictions enabled'}))
      } else {
          let supensionEndDate = new Date(this.state.suspendedOn)
          supensionEndDate.setHours(supensionEndDate.getHours(),supensionEndDate.getMinutes()+this.props.suspendProperties.duration,0,0);
          var msDiff = (new Date(supensionEndDate).getTime() - new Date().getTime());    
          let minutes = Math.floor((msDiff / 1000 / 60) % 60);
          let seconds = Math.floor((msDiff / 1000) % 60);    
          console.log(minutes + ' ' +seconds)
          if (minutes >= 0 && seconds >= 0) {
              if (seconds < 10) {
                seconds = "0" + seconds
              }
              let textSupendedDate = "Restrictions suspended for "+ minutes + ":" + seconds
              this.setState(state => ({
                suspendedDate: textSupendedDate
              }));
          } else {           
            this.setState(state => ({isSuspended: false, suspendButtonDisabled: ''}));      
          }
      } 
  }

  handleClose() {
    this.setState({showModal: false})
  }

  handleShow() {
    this.setState({showModal: true})  
  }

  suspend() {
    // Suspend
      const suspendEndpoint = this.props.serviceURL + '/suspend'
      console.log(suspendEndpoint)
      const payload = '{"duration":"' + this.props.suspendProperties.duration + '"}'

      fetch(suspendEndpoint, {method: 'POST', body: AES.Encrypt(payload, this.props.JAKO, this.props.OJAK)})
      .then(data => {
        this.setState(state => ({isSuspended: true, suspendButtonDisabled: true, suspendedOn: new Date()}))
        // log outcome
        const payload = {
          date: new Date().toLocaleDateString() + " " + new Date().toLocaleTimeString(),
          user: this.props.userObj.username,
          checkbox: this.props.logModule,
          description: "suspended restrictions from",
          computer: this.state.host
        }
        fetch('/api/activity', {method: 'POST', headers: {'Accept': 'application/json', 'Content-Type': 'application/json'}, body: JSON.stringify(payload)})

      })
      .catch(error => {
        console.error('There has been a problem with your fetch operation:', error);
      }); 

      this.handleClose()
  }

  resume() {
    // Unsuspend
      const unsuspendEndpoint = this.props.serviceURL + '/unsuspend'

      fetch(unsuspendEndpoint)
      .then(data => {
        this.setState(state => ({isSuspended: false, suspendButtonDisabled: false}))
        // log outcome
        log(this.props.userObj.username, this.props.logModule, " unsuspended restrictions from ", this.state.host)     
      })
      .catch(error => {
        console.error('There has been a problem with your fetch operation:', error);
      }); 

      this.handleClose()
  }

  render() {
    
    return (
        
      (this.props.summaryDetails != null) ? (
        <div>
          <table width="300">
            <tbody>
              <tr>
                <td align="left">
                <Alert variant="secondary">	
                  <table>
                    <tbody>
                      <tr>
                        <td>
                          <svg width="3em" height="3em" viewBox="0 0 16 16" className="bi bi-lock" fill="currentColor" xmlns="http://www.w3.org/2000/svg">
                          <path fillRule="evenodd" d="M11.5 8h-7a1 1 0 0 0-1 1v5a1 1 0 0 0 1 1h7a1 1 0 0 0 1-1V9a1 1 0 0 0-1-1zm-7-1a2 2 0 0 0-2 2v5a2 2 0 0 0 2 2h7a2 2 0 0 0 2-2V9a2 2 0 0 0-2-2h-7zm0-3a3.5 3.5 0 1 1 7 0v3h-1V4a2.5 2.5 0 0 0-5 0v3h-1V4z"/>
                          </svg>
                        </td>
                        <td>
                        <h2>Restrictions</h2>
                        </td>
                      </tr>
                    </tbody>
                  </table>							 					
                </Alert>
                </td>
              </tr>
              <tr>							
                <td>
                  {
                    (this.state.suspendedDate == 'No restrictions applied') ? 
                    <Alert variant="info"><strong>{this.state.suspendedDate}</strong></Alert>
                    : (this.state.suspendedDate == 'Restrictions enabled') ?
                    <Alert variant="success"><strong>{this.state.suspendedDate}</strong></Alert> 
                    : <Alert variant="warning"><strong>{this.state.suspendedDate}</strong></Alert> 
                  }
                              
                </td>									
              </tr>	
              <tr>							
                <td>
                  <Table striped hover size="sm">
                    <tbody>
                      <tr><td><strong>Available core restrictions:</strong></td><td>{this.props.summaryDetails.Summary.Restrictions.TotalRestrictions}</td></tr>
                      <tr><td><strong>Applied core restrictions:</strong></td><td>{this.props.summaryDetails.Summary.Restrictions.Totals[0].value}</td></tr>
                      <tr><td><strong>Available system keys:</strong></td><td>{this.props.summaryDetails.Summary.SystemKeys.TotalRestrictions}</td></tr>
                      <tr><td><strong>Applied system keys:</strong></td><td>{this.props.summaryDetails.Summary.SystemKeys.Totals[1].value}</td></tr>
                      <tr><td><strong>Allowed applications:</strong></td><td>{this.props.summaryDetails.Summary.Apps.AppsList.length}</td></tr>
                    </tbody>
                  </Table>             
                </td>									
              </tr>
              
              <tr>
                <td>                
                {!this.state.isSuspended ?
                  <Button id="suspend_restrictions" variant="outline-info" size="lg" disabled={this.state.suspendButtonDisabled} block onClick={this.handleShow}> 
                      Suspend Restrictions
                  </Button>
                  :
                  <Button id="resume_restrictions" variant="outline-info" size="lg" block onClick={this.resume}> 
                      Resume Restrictions
                  </Button>
                  }
                </td>
              </tr>		
            </tbody>			
					</table> 
          
          <Modal show={this.state.showModal} onHide={this.handleClose} aria-labelledby="contained-modal-title-vcenter" centered>
            <Modal.Header>
              <Modal.Title>Suspend restrictions</Modal.Title>
            </Modal.Header>
            <Modal.Body>Are you sure you want to suspend the restrictions temporarily for {this.state.duration} minutes?</Modal.Body>
            <Modal.Footer>
              <Button id="btn-cancel-suspend-restrictions" variant="secondary" onClick={this.handleClose}>
                Cancel
              </Button>
              <Button id="btn-apply-suspend-restrictions" variant="primary" onClick={this.suspend}>
                Yes
              </Button>
            </Modal.Footer>
          </Modal>
          </div>
        ) : ''
        
        
        
        );
        
  } 
}


export default SuspendRestrictions;