import {Row, Col, Alert, Table, Jumbotron, Container, Button, Modal, Image, Navbar, Spinner, OverlayTrigger, Tooltip} from 'react-bootstrap'

import AES from '../../lib/aes'
import log from '../../lib/log'
import SmsIcon from '@material-ui/icons/Sms';
import Form from 'react-bootstrap/Form'
import FormControl from 'react-bootstrap/FormControl'
import PhotoCameraIcon from '@material-ui/icons/PhotoCamera';
import IconButton from '@material-ui/core/IconButton'
import { VerticalAlignCenter } from '@material-ui/icons';


class UserDetails extends React.Component {
  constructor(props) {
    super(props);
    // set state variables
    if (this.props.sessionList != null) {
      this.state = {
        sessionList: this.props.sessionList, 
        screenshotURL: '', 
        user: '',
        screenshot: this.props.screenshot,
        showModal: false,
        showErrorModal: false,
      };
    } else {
      this.state = {
        sessionList: []
      }
    }
    console.log(this.props.sessionList)
    this.handleClose = this.handleClose.bind(this);
    this.handleShow = this.handleShow.bind(this);
    this.handleErrorClose = this.handleErrorClose.bind(this);
    this.handleErrorShow = this.handleErrorShow.bind(this); 
    this.handleModalMessageShow = this.handleModalMessageShow.bind(this);   
    this.handleSendMessage = this.handleSendMessage.bind(this);   
    this.handleChange = this.handleChange.bind(this);   
  }

  componentDidMount() {
    // this.interval = setInterval(() => this.checkLogonUsers(), 10000);
		var getUrl = window.location;		
		console.log(getUrl.host)
		var hostIP = getUrl.host
		this.setState(state => ({host: hostIP}))    
  }

  // componentWillUnmount() {
  //   clearInterval(this.interval);
  // }  

  checkLogonUsers() {
    const logonUsersEndpoint = this.props.serviceURL + '/logonuserdetails'
    console.log(logonUsersEndpoint)

    fetch(logonUsersEndpoint)
    .then(response => response.text())
    .then(response => {
      let responseDecoded = JSON.parse(AES.Decrypt(response, this.props.JAKO, this.props.OJAK))
      console.log(responseDecoded.sessions)
      this.setState({sessionList: responseDecoded.sessions})
    })      
    .catch(error => {
      console.error('There has been a problem with your fetch operation:', error);
    });       
  }

  handleClose() {
    this.setState({showModal: false, showMessageModal: false, showSuccessMessageModal: false})
  }

  handleShow(e) {
    const screenshotEndpoint = this.props.serviceURL + '/takescreenshot'
    let user = e.target.id
    if (!user) {
      user = e.target.parentNode.id;
    }    
    let payload = {'user': user}
    this.setState({
      isLoading: true,
      user: user
    })
    // console.log(payload)
    // console.log(JSON.stringify(payload))
    payload = AES.Encrypt(JSON.stringify(payload), this.props.JAKO, this.props.OJAK)
    fetch(screenshotEndpoint, {method: 'POST', body: payload})
    .then(response => response.text())
    .then(response => {
      console.log(response)
      let responseDecoded = JSON.parse(AES.Decrypt(response, this.props.JAKO, this.props.OJAK))
      console.log(responseDecoded)
      this.setState({
        screenshotURL: "data:image/png;base64," +  responseDecoded.base64String,
        isLoading: false
      })
      // log outcome
      log(this.props.user.username, this.props.logModule, " took a screenshot from ", this.state.host)
    })
    .catch(error => {
      this.handleClose()
      this.handleErrorShow()
      console.log('There has been a problem with your fetch operation: ' + error);
    });               

    this.setState({
        showModal: true,
        user: user,
    })  
  }

  handleModalMessageShow(e){
    let user = e.target.id
    if (!user) {
      user = e.target.parentNode.id;
    }
    this.setState({showMessageModal: true, user: user})  
  }

  handleSendMessage() {

    if (this.state.message.length > 0) {
      const userNotificationEndpoint = this.props.serviceURL + '/usernotification'
      let payload = {
        'User': this.state.user, 
        'Message': this.state.message
      }

      payload = AES.Encrypt(JSON.stringify(payload), this.props.JAKO, this.props.OJAK)
      fetch(userNotificationEndpoint, {method: 'POST', body: payload})
      .then(response => {
        if (response.status == '500') {
          throw new Error("Error while importing file");
        } else {
          // log outcome
          log(this.props.user.username, this.props.logModule, " sent message to " + this.state.user, this.state.host)
        }
      })
      .catch(error => {
        this.handleClose()
        this.handleErrorShow()
        console.error('There has been a problem with your fetch operation:', error);
      });               

      this.setState({showSuccessMessageModal: true, showMessageModal: false})  
    }
  }

	handleChange(e) {
    let value = e.target.value.substr(0,97)
    this.setState(state => ({message: value}))	
    console.log(value)
	}    

  handleErrorClose() {
    this.setState({showErrorModal: false})
  }

  handleErrorShow() {
    this.setState({showErrorModal: true})  
  } 

  render() {
        
    let divScreenshot
    if (this.state.isLoading) {
      divScreenshot = <Spinner animation="border" variant="secondary" />;
    } else {
      divScreenshot = <Image src={this.state.screenshotURL} fluid/>
    }
    return (
      
           <div>    
        <Table striped hover size="sm">
        <thead>
            <tr>
                <th>User</th>							
                <th>Session type</th>	
                <th>Login date</th>	
                <th></th>
                <th></th>					
            </tr>
            </thead>
            <tbody>
            { this.state.sessionList.map((session, index) => 
                <tr key={index}>
                    <td className="align-middle">{session.user}</td>
                    <td className="align-middle">{session.LogonType}</td>
                    <td className="align-middle">{new Date(session.sessionStarted).toLocaleDateString(this.props.locale)} {new Date(session.sessionStarted).toLocaleTimeString(this.props.locale)}</td>
                    <td>
                    {session.LogonType != "Interactive" ?                    
                        <IconButton id={session.user} onClick={this.handleModalMessageShow} >
                            <SmsIcon id={session.user}  /> 
                        </IconButton>                    
                    : ''}
                    </td>
                    <td>
                    {session.LogonType != "Interactive" ?                                            
                        <OverlayTrigger placement="bottom" overlay={<Tooltip id="button-tooltip-2">Take screenshot</Tooltip>} >          
                            <IconButton id={session.user} variant="outline" onClick={this.handleShow}>
                            <PhotoCameraIcon id={session.user}  />
                            </IconButton>
                        </OverlayTrigger>                                                              
                    : ''}
                    </td>
                </tr>
            )}
        </tbody>
       </Table>
       <Modal show={this.state.showModal} onHide={this.handleClose} aria-labelledby="contained-modal-title-vcenter" centered>
              <Modal.Header>
                <Modal.Title>{this.state.user}</Modal.Title>
              </Modal.Header>
              <Modal.Body>
                Screenshot         
                <Container>
                <div style={{
                  marginBottom: '1%'
                }}>
                  {divScreenshot}
                  </div>
                </Container>
              </Modal.Body>
              <Modal.Footer>
                <Navbar>
                  <Navbar.Brand href={this.state.screenshotURL} download>
                    <Button id="btn-download-screenshot" variant="primary" >
                      Download
                    </Button>
                  </Navbar.Brand>
                </Navbar>
                <Button id="btn-close-screenshot" variant="secondary" onClick={this.handleClose}>
                  Close
                </Button>                
              </Modal.Footer>
            </Modal>
        <Modal show={this.state.showErrorModal} onHide={this.handleErrorClose} aria-labelledby="contained-modal-title-vcenter" centered>
            <Modal.Header>
                <Modal.Title>Ops, something didn't work as expected</Modal.Title>
            </Modal.Header>
            <Modal.Body>There has been an error when processing your request, please try again or contact support</Modal.Body>
            <Modal.Footer>
                <Button variant="secondary" onClick={this.handleErrorClose}>
                Close
                </Button>
            </Modal.Footer>
        </Modal>        
        <Modal show={this.state.showMessageModal} onHide={this.handleClose} aria-labelledby="contained-modal-title-vcenter" centered>
            <Modal.Header>
                <Modal.Title>Enter the message to be sent to {this.state.user}</Modal.Title>
            </Modal.Header>
            <Modal.Body>
            <Form.Control name="message" placeholder="Enter message" defaultValue={this.state.message} onChange={this.handleChange} />
            </Modal.Body>
            <Modal.Footer>
                <Button variant="primary" onClick={this.handleSendMessage}>
                Send
                </Button>                
                <Button variant="secondary" onClick={this.handleClose}>
                Close
                </Button>
            </Modal.Footer>
        </Modal>   
        <Modal show={this.state.showSuccessMessageModal} onHide={this.handleClose} aria-labelledby="contained-modal-title-vcenter" centered>
              <Modal.Header>
                  <Modal.Title>Message notification</Modal.Title>
              </Modal.Header>
              <Modal.Body>The message has been successfully sent to {this.state.user}</Modal.Body>
              <Modal.Footer>
                  <Button variant="secondary" onClick={this.handleClose}>
                  Close
                  </Button>
              </Modal.Footer>
            </Modal>
       </div>
      
    );
  } 
}

export default UserDetails;