import Header from './layouts/Header';
// import RestrictionsContainer from './layouts/RestrictionsContainer'
import dynamic from 'next/dynamic'
const RestrictionsContainer = dynamic(() => import('./layouts/RestrictionsContainer'))
import { withIronSession } from "next-iron-session";
import AES from '../lib/aes'
import { Alert, Container, Badge} from 'react-bootstrap'
import { Base64 as DefaultPath} from 'js-base64';

export const getServerSideProps = withIronSession(
	async ({ req, res }) => {
		// log module
		const logModule = 'restrictions';
		var user = req.session.get("user");
		var permissions = req.session.get("permissions");

		if (!user) { 
			return { 
        redirect: {
          destination: '/',
          permanent: false,
        }
      };
		} else if (permissions.permissions.indexOf(logModule) < 0 ) {
			res.statusCode = 403;
			res.end();
			return { props: {} };
		} else {			
      // fetch data from config file and create state for all the controllers
      require('dotenv').config()
      const serviceURL = process.env.SERVICE_URL;
			const JAKO = DefaultPath.decode(process.env.JAKO);
      const OJAK = DefaultPath.decode(process.env.OJAK);

      let regedit = require('regedit')
      await regedit.arch.list64('HKLM\\SOFTWARE\\LabLock\\KAgentService')
      .on('data', function(entry) {
        process.env.PORT = entry.data['values']['Port']['value']
        var os = require("os");
        var hostname = os.hostname();
        process.env.SERVICE_URL = 'http://' + hostname + ':' + entry.data['values']['Port']['value']
        console.log(process.env.PORT)
      })
      .on('finish', function () {
        console.log('list operation finished')
      })
      .on('error', function () {
        console.log('Not a Windows OS')
      })    
      console.log(process.env.SERVICE_URL)

      // Retrieve restrictions
      const res = await fetch(serviceURL + '/settings')
      var resEncoded = await res.text()
      let restrictions = await JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
      // console.log(restrictions)
      // Retrieve configurations file
      const res2 = await fetch(serviceURL + '/configurations')
      resEncoded = await res2.text()
      let components = await JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
      // console.log(components.configs)
      // Retrieve syskeys file
      const res3 = await fetch(serviceURL + '/syskeys')
      resEncoded = await res3.text()
      let syskeys = await JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
      // console.log(syskeys)  
      // Retrieve syskeys UI
      const res4 = await fetch(serviceURL + '/syskeysconfig')
      resEncoded = await res4.text()
      let syskeysElements = await JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
      // console.log(syskeysElements)
      // Retrieve syskeys UI
      const res5 = await fetch(serviceURL + '/startup')
      resEncoded = await res5.text()
      let startupPrograms = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
      // console.log(startupPrograms)

      // Retrieve firewall rules
      const firewallRulesJson = await fetch(serviceURL + '/windowsfirewall')
      resEncoded = await firewallRulesJson.text()
      const firewallRules = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
      //  console.log(firewallRules)

      // Retrieve allowed applications
      const applicationsAllowedJson = await fetch(serviceURL + '/appblocker')
      resEncoded = await applicationsAllowedJson.text()      
      const applications = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
      // console.log(applicationsAllowedJson)

      // Retrieve computer details
			const res1 = await fetch(serviceURL + '/computerdetails')
			var resEncoded = await res1.text()
			const computerDetails = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))

			const userObj = user.user

      return {
        props: {
          restrictions,
          components,
          computerDetails,
          syskeys,
          syskeysElements,
          startupPrograms,
          applications,
          firewallRules,
          serviceURL,
          userObj,
          permissions,
          JAKO,
          OJAK
        }
      }
    }
	},
	{
		cookieName: process.env.SESSION_COOKIE,
		cookieOptions: {
			secure: process.env.NODE_ENV === "production" ? true : false,
			maxAge: process.env.SESSION_TIMEOUT * 60
		},
		password: process.env.APPLICATION_SECRET
	}
);

const Index = ( props ) => (

  <>
    <Header user={props.userObj} permissions={props.permissions} serviceURL={props.serviceURL} JAKO={props.JAKO} OJAK={props.OJAK} />

    <Container>
      <Alert variant="info">				
        <h1>Restrictions</h1>
        <p>
					<Badge variant="dark">
  					<em>LabLock - Version: {props.computerDetails.LabLockVersion}</em>
					</Badge>
        </p>
     </Alert>
    </Container>

    <RestrictionsContainer {...props}/>

  </>  
)

export default Index