import Header from './layouts/Header';
import SettingsContainer from './layouts/SettingsContainer'
import { withIronSession } from "next-iron-session";
import AES from '../lib/aes'
import { Alert, Container, Badge} from 'react-bootstrap'
import { Base64 as DefaultPath} from 'js-base64';


export const getServerSideProps = withIronSession(
	async ({ req, res }) => {
		// log module
		const logModule = 'settings';
		var user = req.session.get("user");
		var permissions = req.session.get("permissions");

		if (!user) { 
			return { 
        redirect: {
          destination: '/',
          permanent: false,
        }
      };
		} else if (permissions.permissions.indexOf(logModule) < 0 ) {
			res.statusCode = 403;
			res.end();
			return { props: {} };
		} else {
      // fetch data from config file and create state for all the controllers
      require('dotenv').config()
      const serviceURL = process.env.SERVICE_URL;
			const JAKO = DefaultPath.decode(process.env.JAKO);
      const OJAK = DefaultPath.decode(process.env.OJAK);

      let regedit = require('regedit')
      await regedit.arch.list64('HKLM\\SOFTWARE\\LabLock\\KAgentService')
      .on('data', function(entry) {
        process.env.PORT = entry.data['values']['Port']['value']
        var os = require("os");
        var hostname = os.hostname();
        process.env.SERVICE_URL = 'http://' + hostname + ':' + entry.data['values']['Port']['value']
        console.log(process.env.PORT)
      })
      .on('finish', function () {
        console.log('list operation finished')
      })
      .on('error', function () {
        console.log('Not a Windows OS')
      })    
      console.log(process.env.SERVICE_URL)
            
      // Retrieve general settings
      const res = await fetch(serviceURL + '/lablocksettings')
      var resEncoded = await res.text()
      const generalSettings = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
      console.log(generalSettings)
      process.env.SESSION_TIMEOUT = generalSettings['sessionExpiration']
        
      // Retrieve AD settings
      const res2 = await fetch(serviceURL + '/ldapsettings')
      resEncoded = await res2.text()
      const ldapSettings = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))      
      console.log(ldapSettings)

      let res1 = await fetch(serviceURL + '/computerdetails')
			resEncoded = await res1.text()
			let computerDetails = JSON.parse(AES.Decrypt(resEncoded, JAKO, OJAK))
  
			const userObj = user.user

      return {
        props: { 
          serviceURL,
          generalSettings,
          ldapSettings,
          computerDetails,
          userObj,
          permissions,
          JAKO,
          OJAK
        }
      }
    }
  },
    {
      cookieName: process.env.SESSION_COOKIE,
      cookieOptions: {
        secure: process.env.NODE_ENV === "production" ? true : false,
        maxAge: process.env.SESSION_TIMEOUT * 60
      },
      password: process.env.APPLICATION_SECRET
    }
  );

const Index = ( props ) => (

  <>
    <Header user={props.userObj} permissions={props.permissions} serviceURL={props.serviceURL} JAKO={props.JAKO} OJAK={props.OJAK} />

    <Container>
      <Alert variant="info">				
        <h1>Settings</h1>
        <p>
					<Badge  variant="dark">
  					<em>LabLock - Version: {props.computerDetails.LabLockVersion}</em>
					</Badge>
        </p>
     </Alert>
    </Container>

    <SettingsContainer {...props}/>

  </>  
)

export default Index